import sys
import os
import argparse


def ale_splitter(rec_file, options):
    with open(rec_file) as f:

        lines = f.readlines()

        stree = lines[2].strip()
        ll = lines[6].strip().split()[-1]
        rates = lines[8].strip().split("\t")[1:]

        n_reconciled_trees = int(lines[9].strip().split()[0])
        reconciled_trees = lines[11:n_reconciled_trees + 11]
        n_of_events = lines[11 + n_reconciled_trees + 1].split("\t")[1:]
        table = lines[11 + n_reconciled_trees + 3:]

    if options[0]:
        with open(rec_file.replace("uml_rec", "stree"), "w") as f:
            f.write(stree.split("\t")[-1])
    if options[1]:
        with open(rec_file.replace("uml_rec", "info"), "w") as f:
            f.write("LL:" + "\t" + ll + "\n")
            f.write("Dp:" + "\t" + rates[0] + "\n")
            f.write("Tp:" + "\t" + rates[1] + "\n")
            f.write("Lp:" + "\t" + rates[2] + "\n")
            f.write("De:" + "\t" + n_of_events[0] + "\n")
            f.write("Te:" + "\t" + n_of_events[1] + "\n")
            f.write("Le:" + "\t" + n_of_events[2] + "\n")
            f.write("Se:" + "\t" + n_of_events[3] + "\n")

    if options[2]:
        with open(rec_file.replace("uml_rec", "recs"), "w") as f:
            for t in reconciled_trees:
                f.write(t)
    if options[3]:
        with open(rec_file.replace("uml_rec", "rec_table"), "w") as f:
            for e in table:
                f.write(e)


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("-i", help="Uml_rec file")
    parser.add_argument("-s", help="Prints species tree to a different file", action='store_true', default=False)
    parser.add_argument("-f",
                        help="Prints info about the family (LogLikelihood, probabilities and total number of events, to a different file",
                        action='store_true', default=False)
    parser.add_argument("-t", help="Prints reconciliation table to a different file", action='store_true',
                        default=False)
    parser.add_argument("-r", help="Prints reconciled gene trees to a different file", action='store_true',
                        default=False)
    args = parser.parse_args()

    if args.i == None:
        print("use python ale_splitter.py -h to see options")
        print("you can run this script just with python ale_splitter -i XXXX.uml_rec")
        exit(0)

    ale_splitter(args.i, [args.s, args.f, args.t, args.r])



import os
import argparse
from ete3 import Tree


iTOL_gene_tree_usage = '''
====================== iTOL_gene_tree example commands ======================

TreeSAK iTOL_gene_tree -tree genes.tree -i gnm_taxon.txt -o gene_taxon.txt
TreeSAK iTOL_gene_tree -txt gene_id.txt -i gnm_taxon.txt -o gene_taxon.txt

=============================================================================
'''


def iTOL_gene_tree(args):

    input_tree_file = args['tree']
    input_txt_file  = args['txt']
    meta_txt        = args['i']
    op_txt          = args['o']
    include_na      = args['na']

    if (input_tree_file is None) and (input_txt_file is None):
        print('Please provide gene id with at least one approach, program exited!')
        exit()

    if os.path.isfile(meta_txt) is False:
        print('Metadata file not found, program exited!')
        exit()

    metadata_dict = dict()
    for each_gnm in open(meta_txt):
        each_gnm_split = each_gnm.strip().split('\t')
        if len(each_gnm_split) == 2:
            gnm_id = each_gnm_split[0]
            meta_value = each_gnm_split[1]
            metadata_dict[gnm_id] = meta_value

    gene_id_set = set()
    if input_tree_file is not None:
        if os.path.isfile(input_tree_file) is False:
            print('Tree file not found, program exited!')
            exit()
        else:
            for leaf in Tree(input_tree_file, format=1):
                gene_id_set.add(leaf.name)

    if input_txt_file is not None:
        if os.path.isfile(input_txt_file) is False:
            print('Txt file not found, program exited!')
            exit()
        else:
            for each_id in open(input_txt_file):
                gene_id_set.add(each_id.strip())

    op_txt_handle = open(op_txt, 'w')
    for gene_id in gene_id_set:
        gnm_id = '_'.join(gene_id.split('_')[:-1])
        gnm_meta = metadata_dict.get(gnm_id, 'na')
        if include_na is True:
            op_txt_handle.write('%s\t%s\n' % (gene_id, gnm_meta))
        else:
            if gnm_meta != 'na':
                op_txt_handle.write('%s\t%s\n' % (gene_id, gnm_meta))
    op_txt_handle.close()

    print('Done!')


if __name__ == '__main__':

    iTOL_gene_tree_parser = argparse.ArgumentParser(usage=iTOL_gene_tree_usage)
    iTOL_gene_tree_parser.add_argument('-i',     required=True,                         help='input metadata')
    iTOL_gene_tree_parser.add_argument('-tree',  required=False, default=None,          help='gene id, in tree file')
    iTOL_gene_tree_parser.add_argument('-txt',   required=False, default=None,          help='gene id, in txt file')
    iTOL_gene_tree_parser.add_argument('-o',     required=True,                         help='output metadata')
    iTOL_gene_tree_parser.add_argument('-na',    required=False, action='store_true',   help='include leaves with na values')
    args = vars(iTOL_gene_tree_parser.parse_args())
    iTOL_gene_tree(args)



import os
import glob
import argparse
from Bio import SeqIO
from ete3 import Tree
from distutils.spawn import find_executable


ALE1_usage = '''
====================================== ALE1 example commands ======================================

TreeSAK ALE1 -i OrthologousGroups.txt -s combined.faa -p oma -m 50 -jst 3 -f -o ALE1_op_dir -bmge
TreeSAK ALE1 -ms s03_marker_seq -msx fa -p marker_set_1 -m 50 -jst 3 -f -o ALE1_op_dir -bmge

===================================================================================================
'''


def check_dependencies(program_list):

    not_detected_programs = []
    for needed_program in program_list:
        if find_executable(needed_program) is None:
            not_detected_programs.append(needed_program)

    if not_detected_programs != []:
        print('%s not found, program exited!' % ','.join(not_detected_programs))
        exit()


def sep_path_basename_ext(file_in):

    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def subset_tree(tree_file_in, leaves_to_keep_list, tree_file_out):

    input_tree = Tree(tree_file_in)
    subset_tree = input_tree.copy()
    subset_tree.prune(leaves_to_keep_list, preserve_branch_length=True)
    if tree_file_out is None:
        return subset_tree.write()
    else:
        subset_tree.write(outfile=tree_file_out)


def get_ortho_to_gene_dict(ortho_groups_txt, og_program):

    ortho_to_gene_dict = dict()
    for each_og in open(ortho_groups_txt):
        if not each_og.startswith('#'):
            og_id = ''
            gene_list = []
            if og_program == 'orthofinder':
                each_og_split = each_og.strip().split(' ')
                og_id = each_og_split[0][:-1]
                gene_list = each_og_split[1:]
            elif og_program == 'oma':
                each_og_split = each_og.strip().split('\t')
                og_id = each_og_split[0]
                group_member_list = each_og_split[1:]
                for each_protein in group_member_list:
                    protein_id = each_protein.split(' ')[0].split(':')[1]
                    gene_list.append(protein_id)
            ortho_to_gene_dict[og_id] = gene_list

    return ortho_to_gene_dict


def ALE1(args):

    orthogroups_op_txt          = args['i']
    combined_faa                = args['s']
    og_program                  = args['p']
    marker_seq_dir              = args['ms']
    marker_seq_ext              = args['msx']
    min_og_genome_num           = args['m']
    js_num_threads              = args['jst']
    force_create_op_dir         = args['f']
    op_dir                      = args['o']
    trim_with_bmge              = args['bmge']
    bmge_trim_model             = args['bmge_m']
    bmge_entropy_score_cutoff   = args['bmge_esc']
    designate_ogs               = []
    to_ignore_ogs_list          = []

    # check dependencies
    check_dependencies(['java', 'blastp', 'mafft-einsi'])

    # specify path to BMGE.jar
    current_file_path = '/'.join(os.path.realpath(__file__).split('/')[:-1])
    pwd_bmge_jar      = '%s/BMGE.jar' % current_file_path

    # define output file name
    get_gene_tree_cmds_txt = '%s_cmds.txt' % op_dir

    # determine the version of iqtree available on the system
    if find_executable('iqtree2'):
        iqtree_exe = 'iqtree2'
    elif find_executable('iqtree'):
        iqtree_exe = 'iqtree'
    else:
        print('iqtree not detected, program exited!')
        exit()

    # create op_dir
    if os.path.isdir(op_dir) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    if (orthogroups_op_txt is not None) and (combined_faa is not None) and (marker_seq_dir is None):

        # get ortho_to_gene_dict
        ortho_to_gene_dict = get_ortho_to_gene_dict(orthogroups_op_txt, og_program)

        # get qualified orthogroups
        qualified_og_set = set()
        for each_ortho in ortho_to_gene_dict:
            ortho_gene_set = ortho_to_gene_dict[each_ortho]
            ortho_gnm_set = set()
            for each_gene in ortho_gene_set:
                gene_gnm = '_'.join(each_gene.split('_')[:-1])
                ortho_gnm_set.add(gene_gnm)
            if len(ortho_gnm_set) >= min_og_genome_num:
                qualified_og_set.add(each_ortho)
        print('The total number of identified orthogroups is %s.' % len(ortho_to_gene_dict))
        print('The number of orthogroups spanning >= %s genomes is %s.' % (min_og_genome_num, len(qualified_og_set)))

        # process qualified OG
        og_to_process = sorted([i for i in qualified_og_set])
        if len(designate_ogs) > 0:
            print('The number of designated OGs to process: %s' % len(designate_ogs))
            og_to_process = designate_ogs

        og_to_process_no_ignored = set()
        for each_og in og_to_process:
            if each_og not in to_ignore_ogs_list:
                og_to_process_no_ignored.add(each_og)

        # read sequence into dict
        gene_seq_dict = dict()
        for each_seq in SeqIO.parse(combined_faa, 'fasta'):
            seq_id = each_seq.id
            gene_seq_dict[seq_id] = str(each_seq.seq)

        # extract gene sequences and prepare commands for building gene tree
        print('Preparing commands and sequence files for building gene trees')
        get_gene_tree_cmds_txt_handle = open(get_gene_tree_cmds_txt, 'w')
        for qualified_og in sorted(og_to_process_no_ignored):
            qualified_og_gene_set = ortho_to_gene_dict[qualified_og]
            qualified_og_gene_faa = '%s/%s.faa' % (op_dir, qualified_og)

            og_aln          = '%s.aln'          % qualified_og
            og_aln_trimmed  = '%s_trimmed.aln'  % qualified_og

            # write out commands
            mafft_cmd       = 'mafft-einsi --thread %s --quiet %s.faa > %s'         % (js_num_threads, qualified_og, og_aln)
            trim_cmd        = 'java -jar %s -i %s -m %s -t AA -h %s -of %s'         % (pwd_bmge_jar, og_aln, bmge_trim_model, bmge_entropy_score_cutoff, og_aln_trimmed)
            iqtree_cmd      = '%s -m LG+G+I -bb 1000 --wbtl -nt %s -s %s -pre %s'   % (iqtree_exe, js_num_threads, og_aln, qualified_og)
            if trim_with_bmge is True:
                iqtree_cmd  = '%s -m LG+G+I -bb 1000 --wbtl -nt %s -s %s -pre %s'   % (iqtree_exe, js_num_threads, og_aln_trimmed, qualified_og)

            if trim_with_bmge is True:
                get_gene_tree_cmds_txt_handle.write('%s; %s; %s\n' % (mafft_cmd, trim_cmd, iqtree_cmd))
            else:
                get_gene_tree_cmds_txt_handle.write('%s; %s\n' % (mafft_cmd, iqtree_cmd))

            # write out sequences
            qualified_og_gene_faa_handle = open(qualified_og_gene_faa, 'w')
            for each_gene in qualified_og_gene_set:
                qualified_og_gene_faa_handle.write('>%s\n' % each_gene)
                qualified_og_gene_faa_handle.write('%s\n' % gene_seq_dict[each_gene])
            qualified_og_gene_faa_handle.close()
        get_gene_tree_cmds_txt_handle.close()

    elif (orthogroups_op_txt is None) and (combined_faa is None) and (marker_seq_dir is not None):

        marker_seq_re = '%s/*.%s' % (marker_seq_dir, marker_seq_ext)
        marker_seq_list = glob.glob(marker_seq_re)

        marker_to_gene_dict = dict()
        for each_file in marker_seq_list:
            _, f_base, _ = sep_path_basename_ext(each_file)
            marker_to_gene_dict[f_base] = set()
            for each_seq in SeqIO.parse(each_file, 'fasta'):
                marker_to_gene_dict[f_base].add(each_seq.id)

        # get qualified orthogroups
        qualified_og_set = set()
        for each_ortho in marker_to_gene_dict:
            ortho_gene_set = marker_to_gene_dict[each_ortho]
            ortho_gnm_set = set()
            for each_gene in ortho_gene_set:
                gene_gnm = '_'.join(each_gene.split('_')[:-1])
                ortho_gnm_set.add(gene_gnm)
            if len(ortho_gnm_set) >= min_og_genome_num:
                qualified_og_set.add(each_ortho)
        print('The total number of identified orthogroups is %s.'       % len(marker_to_gene_dict))
        print('The number of orthogroups spanning >= %s genomes is %s.' % (min_og_genome_num, len(qualified_og_set)))

        # process qualified OG
        og_to_process = sorted([i for i in qualified_og_set])
        if len(designate_ogs) > 0:
            print('The number of designated OGs to process: %s' % len(designate_ogs))
            og_to_process = designate_ogs

        og_to_process_no_ignored = set()
        for each_og in og_to_process:
            if each_og not in to_ignore_ogs_list:
                og_to_process_no_ignored.add(each_og)

        # extract gene sequences and prepare commands for building gene tree
        print('Preparing commands for building gene trees')
        get_gene_tree_cmds_txt_handle = open(get_gene_tree_cmds_txt, 'w')
        for qualified_og in sorted(og_to_process_no_ignored):

            # copy sequence file into output directory
            os.system('cp %s/%s.%s %s/' % (marker_seq_dir, qualified_og, marker_seq_ext, op_dir))

            qualified_og_aln          = '%s.aln'          % qualified_og
            qualified_og_aln_trimmed  = '%s_trimmed.aln'  % qualified_og

            # write out commands
            mafft_cmd  = 'mafft-einsi --thread %s --quiet %s.%s > %s'               % (js_num_threads, qualified_og, marker_seq_ext, qualified_og_aln)
            trim_cmd   = 'java -jar %s -i %s -m %s -t AA -h %s -of %s'              % (pwd_bmge_jar, qualified_og_aln, bmge_trim_model, bmge_entropy_score_cutoff, qualified_og_aln_trimmed)
            iqtree_cmd = '%s -m LG+G+I -bb 1000 --wbtl -nt %s -s %s -pre %s'        % (iqtree_exe, js_num_threads, qualified_og_aln, qualified_og)
            if trim_with_bmge is True:
                iqtree_cmd = '%s -m LG+G+I -bb 1000 --wbtl -nt %s -s %s -pre %s'    % (iqtree_exe, js_num_threads, qualified_og_aln_trimmed, qualified_og)

            if trim_with_bmge is False:
                get_gene_tree_cmds_txt_handle.write('%s; %s\n' % (mafft_cmd, iqtree_cmd))
            else:
                get_gene_tree_cmds_txt_handle.write('%s; %s; %s\n' % (mafft_cmd, trim_cmd, iqtree_cmd))

        get_gene_tree_cmds_txt_handle.close()

    print('Sequece files exported to %s.'                       % op_dir)
    print('Commands for inferring gene tree exported to %s.'    % get_gene_tree_cmds_txt)
    print('Done!')


if __name__ == '__main__':

    ALE1_parser = argparse.ArgumentParser()
    ALE1_parser.add_argument('-i',          required=False, default=None,          help='orthologous groups, either from orthofinder or oma')
    ALE1_parser.add_argument('-s',          required=False, default=None,          help='sequence file, e.g., combined.faa')
    ALE1_parser.add_argument('-ms',         required=False, default=None,          help='input is a folder holds the sequence of each marker')
    ALE1_parser.add_argument('-msx',        required=False, default='fa',          help='file extension of marker sequence file, default: fa')
    ALE1_parser.add_argument('-p',          required=True,                         help='orthologous identification program, orthofinder or oma')
    ALE1_parser.add_argument('-m',          required=False, type=int, default=50,  help='min_og_genome_num, default: 50')
    ALE1_parser.add_argument('-bmge',       required=False, action="store_true",   help='trim MSA with BMGE, default no trimming')
    ALE1_parser.add_argument('-bmge_m',     required=False, default='BLOSUM30',    help='BMGE trim model, default: BLOSUM30')
    ALE1_parser.add_argument('-bmge_esc',   required=False, default='0.55',        help='BMGE entropy score cutoff, default: 0.55')
    ALE1_parser.add_argument('-o',          required=True,                         help='output dir, i.e., OMA working directory')
    ALE1_parser.add_argument('-jst',        required=False, type=int, default=3,   help='number of threads specified in job script, default: 3')
    ALE1_parser.add_argument('-f',          required=False, action="store_true",   help='force overwrite')
    args = vars(ALE1_parser.parse_args())
    ALE1(args)



import os
import argparse


ALE2RTC_usage = '''
================================ ALE2RTC example commands ================================

TreeSAK ALE2RTC -h

==========================================================================================
'''


def ALE2RTC():

    pass

    # file_in     = args['i']
    # op_dir      = args['o']


# if __name__ == '__main__':
#
#     ALE2RTC_parser = argparse.ArgumentParser()
#     ALE2RTC_parser.add_argument('-i',     required=True,                   help='the file "out" generated by MCMCTree')
#     ALE2RTC_parser.add_argument('-o',     required=True,                   help='output directory, which will be the input to the pRTC module (-rrtc)')
#     args = vars(ALE2RTC_parser.parse_args())
#     ALE2RTC(args)

ALE2RTC()



import argparse
from ete3 import Tree


mcmcTC_usage = '''
===================== mcmcTC example commands =====================

TreeSAK mcmcTC -i in.tree -o out.tree -tc time_constraints.txt

# Format of constraint file (tab separated columns)
IMG2264867070_yang,GCF900696045_1_yang	3.46-4.38	Archaeal root
GCF000015225_1_yang,GCF000007225_1_yang	-2.32	Oxygen Age Constraint, Thermoproteales
GCF000213215_1_yang,GCA000024305_1_yang	-2.32	Oxygen Age Constraint, Sulfolobales
GCF000152265_2_yang,GCF000195915_1_yang	-2.32	Oxygen Age Constraint, Thermoplasma
GCF000376445_1_yang,GCF000172995_2_yang	-1.579	Chitin Age Constraint, Halobacteriales
GCF000195935_2_yang,GCF000151105_2_yang	-1.579	Chitin Age Constraint, Thermococcales
GCA000802205_2_yang,GCA000200715_1	0.75-1.49	HGT from Viridiplantae to Thaumarchaeota

===================================================================
'''


def mcmcTC(args):

    tree_file_in        = args['i']
    time_constraint_txt = args['tc']
    tree_file_out       = args['o']

    constraint_set = set()
    constraint_dict = dict()
    not_recognizable_time_constraint_set = set()
    for each_constraint in open(time_constraint_txt):
        each_constraint_split = each_constraint.strip().split('\t')
        leaf_ids = each_constraint_split[0]
        provided_age = each_constraint_split[1]

        str_to_add = ''
        if provided_age.startswith('-'):
            str_to_add = '<%s' % provided_age[1:]
        elif provided_age.endswith('-'):
            str_to_add = '>%s' % provided_age[:-1]
        elif '-' in provided_age:
            provided_age_split = provided_age.split('-')
            str_to_add = '>%s<%s' % (provided_age_split[0], provided_age_split[1])
        else:
            not_recognizable_time_constraint_set.add(provided_age)

        constraint_dict[leaf_ids] = str_to_add
        constraint_set.add(str_to_add)

    if len(not_recognizable_time_constraint_set) > 0:
        print('Format of the following time constraints are not recognizable, program exited')
        print(','.join(not_recognizable_time_constraint_set))
        exit()

    # read in tree
    tree_in = Tree(tree_file_in, quoted_node_names=True, format=1)

    # add time constraints as node name
    for each_node in constraint_dict:
        node_age = constraint_dict[each_node]
        node_split = each_node.split(',')
        current_lca = tree_in.get_common_ancestor(node_split)
        current_lca.add_features(custom_label=node_age)
        current_lca.name = node_age

    tree_out_str = tree_in.write(format=1)

    # remove branch length of 1
    tree_out_str = tree_out_str.replace(':1', '')

    # quote constraint strings
    for each_constraint in constraint_set:
        tree_out_str = tree_out_str.replace(each_constraint, ("'%s'" % each_constraint))

    # remove the most outside parenthesis
    tree_out_str = tree_out_str[1:].replace(');', ';')

    # write tree to file
    with open(tree_file_out, 'w') as tree_file_out_handle:
        tree_file_out_handle.write('%s\t1\n' % len(tree_in.get_leaves()))
        tree_file_out_handle.write(tree_out_str)


if __name__ == '__main__':

    mcmcTC_parser = argparse.ArgumentParser()
    mcmcTC_parser.add_argument('-i',   required=True,  help='input tree')
    mcmcTC_parser.add_argument('-o',   required=True,  help='output tree')
    mcmcTC_parser.add_argument('-tc',  required=True,  help='time constraint file')
    args = vars(mcmcTC_parser.parse_args())
    mcmcTC(args)



import itertools


def prep_mcmctree_ctl(ctl_para_dict, mcmctree_ctl_file):

    with open(mcmctree_ctl_file, 'w') as ctl_file_handle:
        ctl_file_handle.write('      finetune = %s\n' % ctl_para_dict.get('seed',           '-1'))
        ctl_file_handle.write('       seqfile = %s\n' % ctl_para_dict['seqfile'])
        ctl_file_handle.write('      treefile = %s\n' % ctl_para_dict['treefile'])
        ctl_file_handle.write('      mcmcfile = %s\n' % ctl_para_dict['mcmcfile'])
        ctl_file_handle.write('       outfile = %s\n' % ctl_para_dict['outfile'])
        ctl_file_handle.write('         ndata = %s\n' % ctl_para_dict.get('ndata',          1))
        ctl_file_handle.write('       seqtype = %s\n' % ctl_para_dict['seqtype'])
        ctl_file_handle.write('       usedata = %s\n' % ctl_para_dict['usedata'])
        ctl_file_handle.write('         clock = %s\n' % ctl_para_dict['clock'])
        ctl_file_handle.write('       RootAge = %s\n' % ctl_para_dict.get('RootAge',        '<1.0'))
        ctl_file_handle.write('         model = %s\n' % ctl_para_dict.get('model',          0))
        ctl_file_handle.write('         alpha = %s\n' % ctl_para_dict.get('alpha',          0.5))
        ctl_file_handle.write('         ncatG = %s\n' % ctl_para_dict.get('ncatG',          4))
        ctl_file_handle.write('     cleandata = %s\n' % ctl_para_dict.get('cleandata',      0))
        ctl_file_handle.write('       BDparas = %s\n' % ctl_para_dict.get('BDparas',        '1 1 0.1'))
        ctl_file_handle.write('   kappa_gamma = %s\n' % ctl_para_dict.get('kappa_gamma',    '6 2'))
        ctl_file_handle.write('   alpha_gamma = %s\n' % ctl_para_dict.get('alpha_gamma',    '1 1'))
        ctl_file_handle.write('   rgene_gamma = %s\n' % ctl_para_dict.get('rgene_gamma',    '1 50 1'))
        ctl_file_handle.write('  sigma2_gamma = %s\n' % ctl_para_dict.get('sigma2_gamma',   '1 10 1'))
        ctl_file_handle.write('      finetune = %s\n' % ctl_para_dict.get('finetune',       '1: .1 .1 .1 .1 .1 .1'))
        ctl_file_handle.write('         print = %s\n' % ctl_para_dict.get('print',          1))
        ctl_file_handle.write('        burnin = %s\n' % ctl_para_dict.get('burnin',         50000))
        ctl_file_handle.write('      sampfreq = %s\n' % ctl_para_dict.get('sampfreq',       5))
        ctl_file_handle.write('       nsample = %s\n' % ctl_para_dict.get('nsample',        50000))


mcmctree_ctl_dict = {'seqfile' : 'concatenated.phy',
                     'treefile': 'deltall75_pa75_rooted_with_calibrations.nwk',
                     'mcmcfile': 'mcmc.txt',
                     'outfile' : 'DateArTree_out.txt',
                     'seqtype' : 2,
                     'usedata' : 3,
                     'clock'   : 3}


prep_mcmctree_ctl(mcmctree_ctl_dict, '/Users/songweizhi/Desktop/aaa.txt')


def get_parameter_combinations(para_to_test_dict):

    para_lol_name = []
    para_lol_value = []
    para_lol_name_with_value = []
    for each_para in sorted(list(para_to_test_dict.keys())):
        para_setting_list_name = []
        para_setting_list_value = []
        para_setting_list_name_with_value = []
        for each_setting in sorted(para_to_test_dict[each_para]):
            name_str = ('%s%s' % (each_para, each_setting)).replace(' ', '_')
            para_setting_list_name.append(each_para)
            para_setting_list_value.append(each_setting)
            para_setting_list_name_with_value.append(name_str)
        para_lol_name.append(para_setting_list_name)
        para_lol_value.append(para_setting_list_value)
        para_lol_name_with_value.append(para_setting_list_name_with_value)

    all_combination_list_name = [p for p in itertools.product(*para_lol_name)]
    all_combination_list_value = [p for p in itertools.product(*para_lol_value)]
    all_combination_list_name_with_value = [p for p in itertools.product(*para_lol_name_with_value)]
    all_combination_list_name_with_value_str = ['_'.join(i) for i in all_combination_list_name_with_value]

    para_dod = dict()
    element_index = 0
    for each_combination in all_combination_list_name_with_value_str:
        current_name_list   = all_combination_list_name[element_index]
        current_value_list  = all_combination_list_value[element_index]
        current_para_dict = dict()
        for key, value in zip(current_name_list, current_value_list):
            current_para_dict[key] = value
        para_dod[each_combination] = current_para_dict
        element_index += 1

    return para_dod


para_to_test_dict = {'clock': [2, 3], 'nsample': [20000, 50000], 'model': [0, 4], 'kappa_gamma': ['6 2', '5 1']}
para_dod = get_parameter_combinations(para_to_test_dict)
print(para_dod)

# all_combination_list_in_str = ['_'.join(i) for i in all_combination_list]
# print(all_combination_list_in_str)
# print(len(all_combination_list_in_str))







import os
import glob
import argparse
from ete3 import Tree
import multiprocessing as mp


ALE2_usage = '''
============================================ ALE2 example commands ===========================================

TreeSAK ALE2 -1 ALE1_op_dir -s genome.treefile -t 10 -f -runALE -docker gregmich/alesuite_new -o ALE2_op_dir

Note: 
Genome names should NOT contain "_", the program will tackle this automatically.

# You can try to add this while building the docker images
--platform linux/arm64/v8

# Only the ufboot files in ALE1_op_dir will be needed in this step.

===============================================================================================================
'''


def sep_path_basename_ext(file_in):

    # separate path and file name
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'

    # separate file basename and extension
    f_base, f_ext = os.path.splitext(file_name)

    return f_path, f_base, f_ext


def subset_tree(tree_file_in, leaves_to_keep_list, tree_file_out):

    input_tree = Tree(tree_file_in)
    subset_tree = input_tree.copy()
    subset_tree.prune(leaves_to_keep_list, preserve_branch_length=True)
    if tree_file_out is None:
        return subset_tree.write()
    else:
        subset_tree.write(outfile=tree_file_out)


def prepare_ale_ip_worker(arg_list):

    ufboot_in   = arg_list[0]
    ufboot_out  = arg_list[1]

    ufboot_out_handle = open(ufboot_out, 'w')
    for each_gene_tree in open(ufboot_in):
        gene_tree_str = each_gene_tree.strip()
        gene_tree_in = Tree(gene_tree_str, format=1)
        for leaf in gene_tree_in:
            leaf_name_split = leaf.name.split('_')
            gnm_id = '_'.join(leaf_name_split[:-1])
            gene_index = leaf_name_split[-1]
            gnm_id_renamed = gnm_id.replace('_', '')
            gene_id_renamed = '%s_%s' % (gnm_id_renamed, gene_index)
            leaf.name = gene_id_renamed
        gene_tree_str_renamed = gene_tree_in.write()
        ufboot_out_handle.write(gene_tree_str_renamed + '\n')
    ufboot_out_handle.close()


def ALE2(args):

    ale1_op_dir             = args['1']
    genome_tree_file_rooted = args['s']
    force_create_ale_wd     = args['f']
    num_threads             = args['t']
    ale2_op_dir             = args['o']
    run_ale                 = args['runALE']
    docker_image            = args['docker']
    run_ale_cmds_txt        = '%s_cmds.txt'     % ale2_op_dir

    ufboot_file_re   = '%s/*.ufboot' % ale1_op_dir
    ufboot_file_list = glob.glob(ufboot_file_re)
    og_to_process_list = []
    for each_ufboot in ufboot_file_list:
        _, ufboot_base, _ = sep_path_basename_ext(each_ufboot)
        og_to_process_list.append(ufboot_base)

    # define file name
    gnm_tree_no_underscore       = 'genome_tree.newick'
    gnm_tree_leaf_rename_txt     = 'genome_tree_leaf_rename.txt'
    gnm_tree_no_underscore_in_wd = '%s/%s' % (ale2_op_dir, gnm_tree_no_underscore)

    # create ale2_op_dir
    if force_create_ale_wd is True:
        if os.path.isdir(ale2_op_dir) is True:
            os.system('rm -r %s' % ale2_op_dir)
    os.system('mkdir %s' % ale2_op_dir)

    # prepare genome tree for running ALE
    gnm_tree_leaf_rename_txt_handle = open(gnm_tree_leaf_rename_txt, 'w')
    gnm_tree_in = Tree(genome_tree_file_rooted, format=1)
    rename_dict = dict()
    for leaf in gnm_tree_in:
        leaf_name = leaf.name
        leaf_name_new = leaf_name.replace('_', '')
        gnm_tree_leaf_rename_txt_handle.write('%s\t%s\n' % (leaf_name_new, leaf.name))
        leaf.name = leaf_name_new
        rename_dict[leaf_name] = leaf_name_new
    gnm_tree_leaf_rename_txt_handle.close()

    gnm_tree_in.write(outfile=gnm_tree_no_underscore_in_wd)

    # prepare gene tree for running ALE
    run_ale_cmds_txt_handle = open(run_ale_cmds_txt, 'w')
    prepare_ale_ip_worker_arg_lol = []
    ale_cmd_list = []
    for qualified_og in og_to_process_list:
        pwd_gene_tree_ufboot = '%s/%s.ufboot' % (ale1_op_dir, qualified_og)
        if os.path.isfile(pwd_gene_tree_ufboot) is False:
            print('%s not found, please build gene tree first!' % pwd_gene_tree_ufboot)
        else:
            pwd_gene_tree_ufboot_in     = '%s/%s.ufboot'                                            % (ale1_op_dir, qualified_og)
            pwd_gene_tree_ufboot_out    = '%s/%s.ufboot'                                            % (ale2_op_dir, qualified_og)

            # get commands for ALEobserve and ALEml_undated
            obtain_ale_file_cmd         = 'ALEobserve %s.ufboot > %s.ALEobserve.log'                % (qualified_og, qualified_og)
            reconciliation_cmd          = 'ALEml_undated %s %s.ufboot.ale > %s.ALEml_undated.log'   % (gnm_tree_no_underscore, qualified_og, qualified_og)
            if docker_image is not None:
                obtain_ale_file_cmd     = 'docker run -v $PWD:$PWD -w $PWD %s %s'                   % (docker_image, obtain_ale_file_cmd)
                reconciliation_cmd      = 'docker run -v $PWD:$PWD -w $PWD %s %s'                   % (docker_image, reconciliation_cmd)

            current_arg_list = [pwd_gene_tree_ufboot_in, pwd_gene_tree_ufboot_out]
            run_ale_cmds_txt_handle.write('%s; %s\n' % (obtain_ale_file_cmd, reconciliation_cmd))
            ale_cmd_list.append('%s; %s\n' % (obtain_ale_file_cmd, reconciliation_cmd))
            prepare_ale_ip_worker_arg_lol.append(current_arg_list)
    run_ale_cmds_txt_handle.close()

    # prepare input files and job script for running ALE with multiprocessing
    print('Preparing files for running ALE with %s cores for %s OGs' % (num_threads, len(prepare_ale_ip_worker_arg_lol)))
    pool = mp.Pool(processes=num_threads)
    pool.map(prepare_ale_ip_worker, prepare_ale_ip_worker_arg_lol)
    pool.close()
    pool.join()

    # run ALE
    if run_ale is True:
        print('running ALE with %s cores for %s OGs' % (num_threads, len(prepare_ale_ip_worker_arg_lol)))
        os.chdir(ale2_op_dir)
        pool = mp.Pool(processes=num_threads)
        pool.map(os.system, ale_cmd_list)
        pool.close()
        pool.join()

    print('Done!')


if __name__ == '__main__':

    ALE2_parser = argparse.ArgumentParser()
    ALE2_parser.add_argument('-1',      required=True,                         help='ALE1 output directory')
    ALE2_parser.add_argument('-s',      required=True,                         help='rooted species tree')
    ALE2_parser.add_argument('-o',      required=True,                         help='output dir, i.e., OMA working directory')
    ALE2_parser.add_argument('-runALE', required=False, action="store_true",   help='run ALE')
    ALE2_parser.add_argument('-docker', required=False, default=None,          help='Docker image, if ALE was installed with Docker, e.g., gregmich/alesuite_new')
    ALE2_parser.add_argument('-f',      required=False, action="store_true",   help='force overwrite')
    ALE2_parser.add_argument('-t',      required=False, type=int, default=6,   help='number of threads, default: 6')
    args = vars(ALE2_parser.parse_args())
    ALE2(args)



import os
import glob
import argparse
from Bio import SeqIO
from ete3 import Tree


ALE6_usage = '''
====================================== ALE6 example commands ======================================

# This module is developed to faa ancestral genomes based on ALE outputs
TreeSAK ALE6 -1 ALE1_op_dir -3 ALE3_op_dir_30 -s species_tree.rooted.treefile -o ALE6_op_dir_30 -n 380 -cog BioSAK_arCOG_wd -kegg BioSAK_KEGG_wd
TreeSAK ALE6 -1 ALE1_op_dir -3 ALE3_op_dir_30 -s species_tree.rooted.treefile -o ALE6_op_dir_30 -n 294,309,380,404
TreeSAK ALE6 -1 ALE1_op_dir -3 ALE3_op_dir_30 -s species_tree.rooted.treefile -o ALE6_op_dir_30 -n interested_nodes.txt

# Needed input files:
-1: faa files
-3: GeneContent.txt, SpeciesTreeRef.newick and Transfer_propensity.txt
-s: the tree used as input for ALE2, to rename leafs back

# To be added:
1. A dereplication step to the produced faa file.

===================================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def get_internal_node_leaves(ale_species_tree_file, internal_node_id):

    ale_species_tree = Tree(ale_species_tree_file, format=1)
    internal_node = ale_species_tree.search_nodes(name=internal_node_id)[0]
    internal_node_leaf_object = internal_node.get_leaves()
    internal_node_leaf_set = set()
    for each_leaf in internal_node_leaf_object:
        internal_node_leaf_set.add(each_leaf.name)

    return internal_node_leaf_set


def rename_leaves(tree_file_in, rename_dict, tree_format, tree_file_out):

    t = Tree(tree_file_in, format=tree_format)
    for leaf in t:
        leaf_name_new = rename_dict.get(leaf.name, leaf.name)
        leaf.name = leaf_name_new
    t.write(format=tree_format, outfile=tree_file_out)


def ALE6(args):

    ale1_op_dir                 = args['1']
    ale3_op_dir                 = args['3']
    op_dir                      = args['o']
    genome_tree_file_rooted     = args['s']
    force_create_op_dir         = args['f']
    interested_internal_nodes   = args['n']
    cog_annotation_wd           = args['cog']
    kegg_annotation_wd          = args['kegg']

    GeneContent_txt             = '%s/GeneContent.txt'          % ale3_op_dir
    SpeciesTreeRef              = '%s/SpeciesTreeRef.newick'    % ale3_op_dir
    transfer_propensity_txt     = '%s/Transfer_propensity.txt'  % ale3_op_dir

    ################################################## define op files #################################################

    _, _, tree_base, tree_ext = sep_path_basename_ext(genome_tree_file_rooted)

    faa_dir                                         = '%s/faa_files'                                    % op_dir
    cog_dir                                         = '%s/annotation_COG'                               % op_dir
    kegg_dir                                        = '%s/annotation_KEGG'                              % op_dir
    cog_df_txt                                      = '%s/annotation_COG.txt'                           % op_dir
    cog_df_desc_txt                                 = '%s/annotation_COG_desc.txt'                      % op_dir
    kegg_df_txt                                     = '%s/annotation_KEGG.txt'                          % op_dir
    kegg_df_desc_txt                                = '%s/annotation_KEGG_desc.txt'                     % op_dir
    fun_transfer_propensity_txt                     = '%s/function_transfer_propensity_weighted.txt'    % op_dir
    genome_tree_file_rooted_with_ale_internal_node  = '%s/%s_with_ALE_internal_nodes.%s'                % (op_dir, tree_base, tree_ext)

    ########################################## get the id of nodes to process ##########################################

    gnm_name_dict_ale_fmt_to_original_fmt = dict()
    for leaf in Tree(genome_tree_file_rooted, format=1):
        leaf_name = leaf.name
        leaf_name_new = leaf_name.replace('_', '')
        leaf.name = leaf_name_new
        gnm_name_dict_ale_fmt_to_original_fmt[leaf_name_new] = leaf_name

    overall_internal_node_set = set()
    line_num_index = 0
    for each_line in open(GeneContent_txt):
        line_num_index += 1
        line_split = each_line.strip().split('\t')
        if line_num_index > 1:
            node_id = line_split[0]
            if '(' not in node_id:
                overall_internal_node_set.add(node_id)

    internal_nodes_to_process = set()
    if interested_internal_nodes is None:
        internal_nodes_to_process = overall_internal_node_set
    else:
        if os.path.isfile(interested_internal_nodes) is False:
            if ',' in interested_internal_nodes:
                internal_nodes_to_process = interested_internal_nodes.split(',')
            else:
                internal_nodes_to_process.add(interested_internal_nodes)
        else:
            for each_node in open(interested_internal_nodes):
                internal_nodes_to_process.add(each_node.strip())

    #################### an addiitonal step (add ALE added internal node names to the rooted tree) #####################

    # create output directory
    if force_create_op_dir is True:
        if os.path.isdir(op_dir) is True:
            os.system('rm -r %s' % op_dir)
    os.system('mkdir %s' % op_dir)

    rename_leaves(SpeciesTreeRef, gnm_name_dict_ale_fmt_to_original_fmt, 1, genome_tree_file_rooted_with_ale_internal_node)

    ####################################################################################################################

    os.system('mkdir %s' % faa_dir)

    if cog_annotation_wd is not None:
        os.system('mkdir %s' % cog_dir)

    if kegg_annotation_wd is not None:
        os.system('mkdir %s' % kegg_dir)

    branch_to_leaf_dict = dict()
    branch_to_content_dict = dict()
    col_header_list = []
    line_num_index = 0
    for each_line in open(GeneContent_txt):
        line_num_index += 1
        line_split = each_line.strip().split('\t')
        if line_num_index == 1:
            col_header_list = line_split
        else:
            branch_id = line_split[0]
            if branch_id in internal_nodes_to_process:
                branch_to_content_dict[branch_id] = []
                branch_child_leaf_set = get_internal_node_leaves(SpeciesTreeRef, branch_id)
                branch_to_leaf_dict[branch_id] = branch_child_leaf_set
                for (id, pa) in zip(col_header_list[1:], line_split[1:]):
                    if pa != '0':
                        branch_to_content_dict[branch_id].append(id)

    branch_to_gene_dict = dict()
    for each_branch in branch_to_content_dict:
        branch_faa = '%s/%s.faa' % (faa_dir, each_branch)
        branch_content = branch_to_content_dict[each_branch]
        branch_child_set = branch_to_leaf_dict[each_branch]
        branch_child_set_original_name = {gnm_name_dict_ale_fmt_to_original_fmt[i] for i in branch_child_set}
        branch_faa_handle = open(branch_faa, 'w')
        branch_to_gene_dict[each_branch] = set()
        for each_prot_family in branch_content:
            each_prot_family_faa = '%s/%s.faa' % (ale1_op_dir, each_prot_family)
            for each_seq in SeqIO.parse(each_prot_family_faa, 'fasta'):
                seq_id = each_seq.id
                seq_gnm = '_'.join(seq_id.split('_')[:-1])
                if seq_gnm in branch_child_set_original_name:
                    branch_faa_handle.write('>%s %s\n' % (each_seq.id, each_prot_family))
                    branch_faa_handle.write('%s\n' % each_seq.seq)
                    branch_to_gene_dict[each_branch].add(each_seq.id)
        branch_faa_handle.close()

    ####################################################################################################################

    # Read in COG annotation results
    fun_to_gene_dict = dict()
    annotation_dict_cog = dict()
    fun_id_to_desc_dict = dict()
    if cog_annotation_wd is not None:

        print('Reading in COG annotation results')
        file_re = '%s/*COG_wd/*_query_to_cog.txt' % (cog_annotation_wd)
        file_list = glob.glob(file_re)

        if len(file_list) == 0:
            print('COG annotation file not detected, program exited!')
            exit()

        for each_file in file_list:
            gnm_id = each_file.split('/')[-1].split('_query_to_cog')[0]
            if gnm_id not in annotation_dict_cog:
                annotation_dict_cog[gnm_id] = dict()
            line_index = 0
            for each_line in open(each_file):
                if line_index > 0:
                    each_line_split = each_line.strip().split('\t')
                    if len(each_line_split) == 4:
                        gene_id = each_line_split[0]
                        cog_id = each_line_split[1]
                        cog_desc = each_line_split[3]
                        annotation_dict_cog[gnm_id][gene_id] = cog_id
                        fun_id_to_desc_dict[cog_id] = cog_desc
                        if cog_id not in fun_to_gene_dict:
                            fun_to_gene_dict[cog_id] = set()
                        fun_to_gene_dict[cog_id].add(gene_id)
                line_index += 1

    # Read in KEGG annotation results
    annotation_dict_kegg = dict()
    if kegg_annotation_wd is not None:

        print('Reading in KEGG annotation results')
        file_re = '%s/*KEGG_wd/*_ko_assignment_ABCD.txt' % (kegg_annotation_wd)
        file_list = glob.glob(file_re)

        if len(file_list) == 0:
            print('KEGG annotation file not detected, program exited!')
            exit()

        for each_file in file_list:
            gnm_id = each_file.split('/')[-1].split('_ko_assignment_ABCD')[0]
            if gnm_id not in annotation_dict_kegg:
                annotation_dict_kegg[gnm_id] = dict()

            line_index = 0
            for each_line in open(each_file):
                if line_index > 0:
                    each_line_split = each_line.strip().split('\t')
                    if len(each_line_split) == 9:
                        gene_id = each_line_split[0]
                        ko_d_id = each_line_split[4][2:]
                        ko_d_desc = each_line_split[8]
                        annotation_dict_kegg[gnm_id][gene_id] = ko_d_id
                        fun_id_to_desc_dict[ko_d_id] = ko_d_desc
                        if ko_d_id not in fun_to_gene_dict:
                            fun_to_gene_dict[ko_d_id] = set()
                        fun_to_gene_dict[ko_d_id].add(gene_id)
                line_index += 1

    cog_dod = dict()
    kegg_dod = dict()
    all_identified_cog_set = set()
    all_identified_kegg_set = set()
    if (cog_annotation_wd is not None) or (kegg_annotation_wd is not None):
        for each_branch in branch_to_gene_dict:
            branch_gene_content = branch_to_gene_dict[each_branch]
            branch_cog_set = set()
            branch_kegg_set = set()
            for each_gene in branch_gene_content:
                gnm_id = '_'.join(each_gene.split('_')[:-1])
                cog_fun = annotation_dict_cog[gnm_id].get(each_gene, 'na')
                kegg_fun = annotation_dict_kegg[gnm_id].get(each_gene, 'na')
                if cog_fun != 'na':
                    branch_cog_set.add(cog_fun)
                    all_identified_cog_set.add(cog_fun)
                if kegg_fun != 'na':
                    branch_kegg_set.add(kegg_fun)
                    all_identified_kegg_set.add(kegg_fun)

            cog_dod[each_branch] = branch_cog_set
            kegg_dod[each_branch] = branch_kegg_set

            # write out annotation
            if len(branch_cog_set) > 0:
                cog_annotation_txt = '%s/%s_COG.txt' % (cog_dir, each_branch)
                cog_annotation_txt_handle = open(cog_annotation_txt, 'w')
                for each_cog in sorted(list(branch_cog_set)):
                    cog_annotation_txt_handle.write('%s\t%s\n' % (each_cog, fun_id_to_desc_dict[each_cog]))
                cog_annotation_txt_handle.close()

            if len(branch_kegg_set) > 0:
                kegg_annotation_txt = '%s/%s_KEGG.txt' % (kegg_dir, each_branch)
                kegg_annotation_txt_handle = open(kegg_annotation_txt, 'w')
                for each_kegg in sorted(list(branch_kegg_set)):
                    kegg_annotation_txt_handle.write('%s\t%s\n' % (each_kegg, fun_id_to_desc_dict[each_kegg]))
                kegg_annotation_txt_handle.close()

    all_identified_cog_list_sorted       = sorted(list(all_identified_cog_set))
    all_identified_kegg_list_sorted      = sorted(list(all_identified_kegg_set))
    all_identified_cog_list_sorted_desc  = [('%s__%s' % (i, fun_id_to_desc_dict[i])) for i in all_identified_cog_list_sorted]
    all_identified_kegg_list_sorted_desc = [('%s__%s' % (i, fun_id_to_desc_dict[i])) for i in all_identified_kegg_list_sorted]

    # write out COG dataframe
    if len(all_identified_cog_set) > 0:
        cog_df_txt_handle = open(cog_df_txt, 'w')
        cog_df_txt_handle.write('\t%s\n' % '\t'.join(all_identified_cog_list_sorted))
        cog_df_desc_txt_handle = open(cog_df_desc_txt, 'w')
        cog_df_desc_txt_handle.write('\t%s\n' % '\t'.join(all_identified_cog_list_sorted_desc))
        for each_branch in sorted(list(cog_dod.keys())):
            branch_cogs = cog_dod[each_branch]
            cog_pa_list = [each_branch]
            for each_cog in all_identified_cog_list_sorted:
                if each_cog in branch_cogs:
                    cog_pa_list.append('1')
                else:
                    cog_pa_list.append('0')
            cog_df_txt_handle.write('\t'.join(cog_pa_list) + '\n')
            cog_df_desc_txt_handle.write('\t'.join(cog_pa_list) + '\n')
        cog_df_txt_handle.close()
        cog_df_desc_txt_handle.close()
        print('Annotation matrix exported to: %s' % cog_df_txt)

    # write out KEGG dataframe
    if len(all_identified_kegg_set) > 0:
        kegg_df_txt_handle = open(kegg_df_txt, 'w')
        kegg_df_txt_handle.write('\t%s\n' % '\t'.join(all_identified_kegg_list_sorted))
        kegg_df_desc_txt_handle = open(kegg_df_desc_txt, 'w')
        kegg_df_desc_txt_handle.write('\t%s\n' % '\t'.join(all_identified_kegg_list_sorted_desc))
        for each_branch in sorted(list(kegg_dod.keys())):
            branch_keggs = kegg_dod[each_branch]
            kegg_pa_list = [each_branch]
            for each_kegg in all_identified_kegg_list_sorted:
                if each_kegg in branch_keggs:
                    kegg_pa_list.append('1')
                else:
                    kegg_pa_list.append('0')
            kegg_df_txt_handle.write('\t'.join(kegg_pa_list) + '\n')
            kegg_df_desc_txt_handle.write('\t'.join(kegg_pa_list) + '\n')
        kegg_df_txt_handle.close()
        kegg_df_desc_txt_handle.close()
        print('Annotation matrix exported to: %s' % kegg_df_txt)

    ################################# get transfer propensity of individual functions ##################################

    print('Getting transfer propensity of individual function')

    # get gene_to_oma_dict
    faa_file_re = '%s/*.faa' % ale1_op_dir
    faa_file_list = glob.glob(faa_file_re)
    gene_to_oma_dict = dict()
    for faa_file in faa_file_list:
        _, _, faa_base, _ = sep_path_basename_ext(faa_file)
        for each_seq in SeqIO.parse(faa_file, 'fasta'):
            seq_id = each_seq.id
            gene_to_oma_dict[seq_id] = faa_base

    # get oma_to_transfer_propensity_dict
    oma_to_transfer_propensity_dict = dict()
    line_index = 0
    for each_oma in open(transfer_propensity_txt):
        if line_index > 0:
            each_oma_split = each_oma.strip().split('\t')
            oma_id = each_oma_split[0]
            transfer_propensity = float(each_oma_split[1])
            oma_to_transfer_propensity_dict[oma_id] = transfer_propensity
        line_index += 1

    fun_transfer_propensity_txt_handle = open(fun_transfer_propensity_txt, 'w')
    fun_transfer_propensity_txt_handle.write('ID\tWeighted_transfer_propensity\tDescription\n')
    oma_weighted_transfer_propensity_dict = dict()
    for fun_id in sorted(list(fun_to_gene_dict.keys())):
        current_fun_gene_set = fun_to_gene_dict[fun_id]
        current_fun_oma_stats_dict = dict()
        for gene_id in current_fun_gene_set:
            gene_oma = gene_to_oma_dict.get(gene_id, 'na')
            if gene_oma not in current_fun_oma_stats_dict:
                current_fun_oma_stats_dict[gene_oma] = 1
            else:
                current_fun_oma_stats_dict[gene_oma] += 1

        total_transfer_propensity = 0
        total_oma_num = 0
        for oma_id in current_fun_oma_stats_dict:
            oma_num = current_fun_oma_stats_dict[oma_id]
            oma_transfer_propensity = oma_to_transfer_propensity_dict.get(oma_id, 'na')
            if oma_transfer_propensity != 'na':
                total_transfer_propensity += (oma_num*oma_transfer_propensity)
                total_oma_num += oma_num

        oma_transfer_propensity_weighted = 'na'
        if total_oma_num != 0:
            oma_transfer_propensity_weighted = total_transfer_propensity/total_oma_num
            oma_transfer_propensity_weighted = float("{0:.3f}".format(oma_transfer_propensity_weighted))

        if oma_transfer_propensity_weighted != 'na':
            oma_weighted_transfer_propensity_dict[fun_id] = oma_transfer_propensity_weighted
            fun_transfer_propensity_txt_handle.write('%s\t%s\t%s\n' % (fun_id, oma_transfer_propensity_weighted, fun_id_to_desc_dict[fun_id]))

    fun_transfer_propensity_txt_handle.close()

    print('Done!')


if __name__ == '__main__':

    ALE6_parser = argparse.ArgumentParser()
    ALE6_parser.add_argument('-1',      required=True,                          help='ALE1 output directory')
    ALE6_parser.add_argument('-3',      required=True,                          help='ALE3 output directory')
    ALE6_parser.add_argument('-s',      required=True,                          help='rooted species tree')
    ALE6_parser.add_argument('-n',      required=False, default=None,           help='interested internal node(s)')
    ALE6_parser.add_argument('-cog',    required=False, default=None,           help='COG annotation results')
    ALE6_parser.add_argument('-kegg',   required=False, default=None,           help='KEGG annotation results')
    ALE6_parser.add_argument('-o',      required=True,                          help='output directory')
    ALE6_parser.add_argument('-f',      required=False, action="store_true",    help='force overwrite')
    args = vars(ALE6_parser.parse_args())
    ALE6(args)



import os
import glob
import argparse
from Bio import SeqIO


filter_rename_ar53_usage = '''
===================================== filter_rename_ar53 example commands =====================================

TreeSAK filter_rename_ar53 -i seq_dir -x fa -g interested_gnms.txt -m interested_marker.txt -o seq_dir_renamed

===============================================================================================================
'''

def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]

    return f_name, f_path, f_base, f_ext


def get_shared_uniq_elements(list_1, list_2):
    shared_set = set(list_1).intersection(list_2)
    list_1_uniq = []
    for e1 in list_1:
        if e1 not in shared_set:
            list_1_uniq.append(e1)
    list_2_uniq = []
    for e2 in list_2:
        if e2 not in shared_set:
            list_2_uniq.append(e2)
    return shared_set, list_1_uniq, list_2_uniq


def filter_rename_ar53(args):

    marker_seq_dir          = args['i']
    marker_seq_ext          = args['x']
    interested_gnm_txt      = args['g']
    interested_marker_txt   = args['m']
    op_dir                  = args['o']
    force_overwrite         = args['f']

    # create output folder
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('%s exist, program exited!' % op_dir)
            exit()
    os.mkdir(op_dir)

    interested_marker_set = set()
    if os.path.isfile(interested_marker_txt) is True:
        for each_marker in open(interested_marker_txt):
            interested_marker_set.add(each_marker.strip())
        if len(interested_marker_set) == 0:
            print('No marker provided in %s, program exited!' % interested_gnm_txt)
            exit()

    interested_gnm_set = set()
    if os.path.isfile(interested_gnm_txt) is True:
        for each_gnm in open(interested_gnm_txt):
            interested_gnm_set.add(each_gnm.strip())
        if len(interested_gnm_set) == 0:
            print('No genome provided in %s, program exited!' % interested_gnm_txt)
            exit()

    marker_seq_re   = '%s/*.%s' % (marker_seq_dir, marker_seq_ext)
    marker_seq_list = [os.path.basename(i) for i in glob.glob(marker_seq_re)]
    marker_seq_id_list = [('.'.join(i.split('.')[:-1])) for i in marker_seq_list]
    if len(marker_seq_list) == 0:
        print('No file found in %s, program exited!' % marker_seq_dir)
        exit()

    marker_to_process = set()
    if len(interested_marker_set) == 0:
        marker_to_process = marker_seq_id_list
    else:
        shared_set, marker_seq_uniq, interested_marker_uniq = get_shared_uniq_elements(marker_seq_id_list, interested_marker_set)
        if len(interested_marker_uniq) > 0:
            print('Sequences for the following interested markers were not found:')
            print(','.join(interested_marker_uniq))
        marker_to_process = shared_set

    for marker_id in marker_to_process:
        pwd_file_in  = '%s/%s.%s' % (marker_seq_dir, marker_id, marker_seq_ext)
        pwd_file_out = '%s/%s.%s' % (op_dir, marker_id, marker_seq_ext)
        pwd_op_file_handle = open(pwd_file_out, 'w')
        for each_seq in SeqIO.parse(pwd_file_in, 'fasta'):
            gnm_id = each_seq.id
            if os.path.isfile(interested_gnm_txt) is False:
                pwd_op_file_handle.write('>%s_XXX\n' % gnm_id)
                pwd_op_file_handle.write('%s\n' % str(each_seq.seq))
            else:
                if gnm_id in interested_gnm_set:
                    pwd_op_file_handle.write('>%s_XXX\n' % gnm_id)
                    pwd_op_file_handle.write('%s\n' % str(each_seq.seq))
        pwd_op_file_handle.close()

    print('Done!')


if __name__ == '__main__':

    filter_rename_ar53_parser = argparse.ArgumentParser()
    filter_rename_ar53_parser.add_argument('-i',   required=True,                          help='sequence folder')
    filter_rename_ar53_parser.add_argument('-x',   required=True,                          help='file extension')
    filter_rename_ar53_parser.add_argument('-g',   required=False, default=None,           help='interested genome, no ext, one id per line')
    filter_rename_ar53_parser.add_argument('-m',   required=False, default=None,           help='interested marker, no ext, one id per line')
    filter_rename_ar53_parser.add_argument('-o',   required=True,                          help='output folder')
    filter_rename_ar53_parser.add_argument('-f',   required=False, action="store_true",    help='force overwrite')
    args = vars(filter_rename_ar53_parser.parse_args())
    filter_rename_ar53(args)



import os
import glob
import argparse


OMA_usage = '''
======================= OMA example commands =======================

TreeSAK OMA -i faa_files -x faa -og og_gnm.txt -o OMA_wd -f -t 32

====================================================================
'''

def sep_path_basename_ext(file_in):

    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(file_name)
    return f_path, f_base, f_ext


def get_default_para_dict():

    default_para_str = '''
    OutputFolder := 'Output';
    ReuseCachedResults := true;
    AlignBatchSize := 1e6;
    MinScore := 181;
    LengthTol := 0.61;
    StablePairTol := 1.81;
    InparalogTol := 3.00;
    ParalogTol := -2.5*StablePairTol;
    VerifiedPairTol := 1.53;
    MinSeqLen := 50;
    UseOnlyOneSplicingVariant := true;
    UseExperimentalHomologousClusters := false;
    QuasiCliquesCutoff := 1.0:
    StableIdsForGroups := false;
    GuessIdType := false;
    DoHierarchicalGroups := 'bottom-up';
    SpeciesTree := 'estimate';
    MinEdgeCompletenessFraction := 0.65;
    ReachabilityCutoff := 0.65;
    MaxTimePerLevel := 1200;  # 20min
    DoGroupFunctionPrediction := true;
    GroupFunctionCutoff := 0.5;
    CladeDefinition := 'default';
    UseEsprit := false;
    DistConfLevel := 2;
    MinProbContig := 0.4;
    MaxContigOverlap := 5;
    MinSeqLenContig := 20;
    MinBestScore := 250;
    '''

    default_para_dict = dict()
    for each_line in default_para_str.split('    '):
        para_line = each_line.replace(' ', '').replace('\n', '').split(';')[0]
        if para_line != '':
            para_line_split = para_line.split(':=')
            default_para_dict[para_line_split[0]] = para_line_split[1]

    return default_para_dict


def OMA(args):

    gnm_dir         = args['i']
    file_ext        = args['x']
    seq_type        = args['st']
    og_gnm_txt      = args['og']
    op_dir          = args['o']
    force_overwrite = args['f']
    num_threads     = args['t']

    # define file name
    pwd_gnm_rename_txt = '%s/rename.txt'     % op_dir
    pwd_parameter_file = '%s/parameters.drw' % op_dir
    oma_input_dir      = '%s/DB'             % op_dir

    # create dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % oma_input_dir)

    # check genome files
    gnm_file_re   = '%s/*.%s' % (gnm_dir, file_ext)
    gnm_file_list = glob.glob(gnm_file_re)
    if len(gnm_file_list) == 0:
        print('No genome detected, program exited!')
        exit()

    # check og_gnm_txt
    if os.path.isfile(og_gnm_txt) is False:
        print('Out group genome id file not detected, program exited!')
        exit()

    # copy genome files into DB folder
    gnm_id_rename_dict = dict()
    rename_list = []
    for each_gnm in gnm_file_list:
        gnm_path, gnm_base, gnm_ext = sep_path_basename_ext(each_gnm)
        gnm_base_renamed = gnm_base.replace('.', '_')
        pwd_gnm_db = '%s/%s.fa' % (oma_input_dir, gnm_base_renamed)
        if gnm_base != gnm_base_renamed:
            rename_list.append('%s\t%s' % (gnm_base, gnm_base_renamed))
        gnm_id_rename_dict[gnm_base] = gnm_base_renamed
        os.system('cp %s %s' % (each_gnm, pwd_gnm_db))

    # write out rename file
    if len(rename_list) > 0:
        pwd_gnm_rename_txt_handle = open(pwd_gnm_rename_txt, 'w')
        for each_e in sorted(rename_list):
            pwd_gnm_rename_txt_handle.write(each_e + '\n')
        pwd_gnm_rename_txt_handle.close()
    else:
        print('Format of file names passed checking')

    # get default_para_dict
    default_para_dict = get_default_para_dict()

    # read in og_gnm_txt
    renamed_og_gnm_list = []
    for each_og_gnm in open(og_gnm_txt):
        og_gnm_renamed = gnm_id_rename_dict[each_og_gnm.strip()]
        renamed_og_gnm_list.append(og_gnm_renamed)

    # write out parameter file
    with open(pwd_parameter_file, 'w') as pwd_parameter_file_handle:

        # write InputDataType line
        if seq_type in ['AA', 'aa', 'Aa']:
            pwd_parameter_file_handle.write("InputDataType := 'AA';\n")
        if seq_type in ['DNA', 'dna', 'Dna']:
            pwd_parameter_file_handle.write("InputDataType := 'DNA';\n")

        # write OutgroupSpecies line
        OutgroupSpecies_value_str = "['%s']" % "', '".join(renamed_og_gnm_list)
        pwd_parameter_file_handle.write("OutgroupSpecies := %s;\n" % OutgroupSpecies_value_str)

        # write out the rest lines
        for each_para in default_para_dict:
            para_value = default_para_dict[each_para]
            pwd_parameter_file_handle.write("%s := %s;\n" % (each_para, para_value))

    # final report
    print('You can run OMA with:')
    print('cd %s' % op_dir)
    print('oma -n %s' % num_threads)
    print('# You may want to customize parameters specified in %s ' % pwd_parameter_file)


if __name__ == '__main__':

    OMA_parser = argparse.ArgumentParser()
    OMA_parser.add_argument('-i',   required=True,                       help='genome folder')
    OMA_parser.add_argument('-x',   required=True,                       help='genome file extension')
    OMA_parser.add_argument('-st',  required=False, default='AA',        help='sequence type, AA or DNA, default: AA')
    OMA_parser.add_argument('-og',  required=True,                       help='outgroup genomes, without file extension')
    OMA_parser.add_argument('-o',   required=True,  default=None,        help='output dir, i.e., OMA working directory')
    OMA_parser.add_argument('-f',   required=False, action="store_true", help='force overwrite')
    OMA_parser.add_argument('-t',   required=False, type=int, default=6, help='number of threads for running OMA, default: 6')
    args = vars(OMA_parser.parse_args())
    OMA(args)



from ete3 import Tree


def root_with_out_group(tree_file, out_group_txt, tree_file_rooted):

    out_group_set = set()
    for each_og in open(out_group_txt):
        out_group_set.add(each_og.strip())

    tre = Tree(tree_file, format=1)
    out_group_lca = tre.get_common_ancestor(out_group_set)
    tre.set_outgroup(out_group_lca)
    tre.write(outfile=tree_file_rooted)


def replace_clades(main_tree, sub_tree, tree_out):

    # read in sub tree
    tre_sub = Tree(sub_tree, format=1)

    # get all leaves in sub tree
    subtree_leaf_name_list = tre_sub.get_leaf_names()

    # read in main tree
    tre_main = Tree(main_tree)

    # remove clades
    lca = tre_main.get_common_ancestor(subtree_leaf_name_list)

    if len(lca.get_leaf_names()) != len(subtree_leaf_name_list):
        print('LCA of subtree leaves in main tree contain extra leaves, program exited!')
        exit()

    lca_p = lca.up
    lca_p.remove_child(lca)
    lca_p.add_child(tre_sub)

    # write out updated tree
    tre_main.write(outfile=tree_out, format=8)


tree_file                           = '/Users/songweizhi/Desktop/777/PA_75_DeltaLL_50_raw.treefile'
out_group_txt                       = '/Users/songweizhi/Desktop/777/out_group.txt'
tree_file_rooted                    = '/Users/songweizhi/Desktop/777/PA_75_DeltaLL_50_rooted.treefile'
eu_tree                             = '/Users/songweizhi/Desktop/777/27.nwk'
rooted_tree_with_time_constraints   = '/Users/songweizhi/Desktop/777/PA_75_DeltaLL_50_rooted_with_time_constraints.treefile'


root_with_out_group(tree_file, out_group_txt, tree_file_rooted)
replace_clades(tree_file_rooted, eu_tree, rooted_tree_with_time_constraints)


root_with_out_group_from_tianhua = '''
from ete3 import Tree
tpath = ''
nodes = ['F7','B7_3','A7']
tre = Tree(tpath)
tre2 = tre.copy()
lca = tre.get_common_ancestor(nodes)
lca_leaves = lca.get_leaf_names()
# intersect = set(lca_leaves).intersection(set(nodes))
ratio = len(nodes)/len(lca_leaves)
if ratio > 0.5:
    tre2.set_outgroup(lca)  # inplace
    tre2.write(outfile='')
else:
    print(ratio)
'''


replace_clade_from_tianhua ='''
## replace clade with a new tree    
tre2 = Tree(tpath2,format=3)  
nodes1 = []
tre = Tree(tpath)
lca = tre.get_common_ancestor(nodes1)
lca_p = lca.up
lca_p.remove_child(lca)
if len(tre2.children)==1:
    lca_p.add_child(tre2.children[0])
else:
    lca_p.add_child(tre2)
tre.write(outfile='',format='')
'''


import argparse
from Bio import SeqIO


recode_usage = '''
============================ recode example commands ============================

TreeSAK recode -i msa.fa -m s4 -o recoded_msa_SR4.fa
TreeSAK recode -i msa.fa -m d4 -o recoded_msa_Dayhoff4.fa
TreeSAK recode -i msa.fa -m d6 -o recoded_msa_Dayhoff6.fa

Note:
This script is modified based on the Recode_aa.py from Anja Spang.
It was used to recode AA to SR4 (s4), Dayhoff4 (d4) or Dayhoff6 (d6) categories.  
Please refer to https://doi.org/10.1038/s41467-020-17408-w for details.

Recoding schemes
1. Selenocysteine will be recoded to a gap.
2. s4: A,G,N,P,S,T = A; C,H,W,Y = C; D,E,K,Q,R = G; F,I,L,M,V = T
3. d4: A,G,P,S,T = A; D,E,N,Q = D; H,K,R = H; F,Y,W,I,L,M,V = F; C = ?
4. d6: A,G,P,S,T = A; D,E,N,Q = D; H,K,R = H; F,Y,W = F; I,L,M,V = I; C = C

=================================================================================
'''


def recode(args):

    msa_in   = args['i']
    msa_out  = args['o']
    category = args['m']

    DH6 = {'-': '-', 'A': 'A', 'G': 'A', 'P': 'A', 'S': 'A', 'T': 'A', 'D': 'D', 'E': 'D', 'N': 'D', 'Q': 'D', 'H': 'H', 'K': 'H', 'R': 'H', 'F': 'F', 'Y': 'F', 'W': 'F', 'I': 'I', 'L': 'I', 'M': 'I', 'V': 'I', 'C': 'C'}
    DH4 = {'-': '-', 'A': 'A', 'G': 'A', 'P': 'A', 'S': 'A', 'T': 'A', 'D': 'D', 'E': 'D', 'N': 'D', 'Q': 'D', 'H': 'H', 'K': 'H', 'R': 'H', 'F': 'F', 'Y': 'F', 'W': 'F', 'I': 'F', 'L': 'F', 'M': 'F', 'V': 'F', 'C': '-'}
    SR4 = {'-': '-', 'A': 'A', 'G': 'A', 'N': 'A', 'P': 'A', 'S': 'A', 'T': 'A', 'C': 'C', 'H': 'C', 'W': 'C', 'Y': 'C', 'D': 'G', 'E': 'G', 'K': 'G', 'Q': 'G', 'R': 'G', 'F': 'T', 'I': 'T', 'L': 'T', 'M': 'T', 'V': 'T'}

    msa_out_handle = open(msa_out, 'w')
    for seq_record in SeqIO.parse(msa_in, "fasta"):
        header = str(seq_record.description).strip()
        seq = str(seq_record.seq).strip()
        new_seq = ''
        for item in seq:
            if category in ['D6', 'd6']:
                if item in DH6:
                    new_seq = new_seq + str(DH6.get(item))
                else:
                    new_seq = new_seq + str('-')
            elif category in ['D4', 'd4']:
                if item in DH4:
                    new_seq = new_seq + str(DH4.get(item))
                else:
                    new_seq = new_seq + str('-')
            elif category in ['S4', 's4']:
                if item in SR4:
                    new_seq = new_seq + str(SR4.get(item))
                else:
                    new_seq = new_seq + str('-')
            else:
                print('Please choose recoding scheme from d4, d6 and s4, program exited!')
                exit()

        msa_out_handle.write(">%s\n%s\n" % (header, new_seq))
    msa_out_handle.close()


if __name__ == '__main__':

    recode_parser = argparse.ArgumentParser(description='Recode amino acids to Dayoff 4, Dayoff 6 or SR4 categories')
    recode_parser.add_argument('-i', required=True,  help='input file')
    recode_parser.add_argument('-m', required=True,  help='recoding scheme, choose from d4, d6 or s4')
    recode_parser.add_argument('-o', required=True,  help='output file')
    args = vars(recode_parser.parse_args())
    recode(args)



import os
import glob
from Bio import SeqIO
from Bio import AlignIO


def catfasta2phy(msa_dir, msa_ext, concatenated_msa_phy, partition_file):

    concatenated_msa_fasta = '%s.fasta' % concatenated_msa_phy
    msa_file_re            = '%s/*.%s'  % (msa_dir, msa_ext)
    msa_file_list          = [os.path.basename(file_name) for file_name in glob.glob(msa_file_re)]
    msa_file_list_sorted   = sorted(msa_file_list)

    complete_gnm_set = set()
    for each_msa_file in msa_file_list:
        pwd_msa = '%s/%s' % (msa_dir, each_msa_file)
        for each_seq in SeqIO.parse(pwd_msa, 'fasta'):
            complete_gnm_set.add(each_seq.id)

    complete_gnm_list_sorted = sorted([i for i in complete_gnm_set])

    # initialize concatenated msa dict
    gnm_to_seq_dict = {i: '' for i in complete_gnm_list_sorted}
    msa_len_dict = dict()
    for each_msa_file in msa_file_list_sorted:
        gene_id = each_msa_file.split('.' + msa_ext)[0]

        # read in msa
        current_msa_len = 0
        current_msa_len_set = set()
        pwd_current_msa = '%s/%s' % (msa_dir, each_msa_file)
        current_msa_seq_dict = dict()
        for each_seq in SeqIO.parse(pwd_current_msa, 'fasta'):
            complete_gnm_set.add(each_seq.id)
            current_msa_seq_dict[each_seq.id] = str(each_seq.seq)
            current_msa_len_set.add(len(each_seq.seq))
            current_msa_len = len(each_seq.seq)

        if len(current_msa_len_set) != 1:
            print('Sequences with different length were found in %s, program exited!' % each_msa_file)
            exit()

        msa_len_dict[gene_id] = current_msa_len

        # add sequence to concatenated msa dict
        for each_gnm in complete_gnm_list_sorted:
            msa_seq = current_msa_seq_dict.get(each_gnm, current_msa_len*'-')
            gnm_to_seq_dict[each_gnm] += msa_seq

    # write out concatenated msa
    concatenated_msa_handle = open(concatenated_msa_fasta, 'w')
    for each_gnm in complete_gnm_list_sorted:
        concatenated_msa_handle.write('>%s\n' % each_gnm)
        concatenated_msa_handle.write('%s\n' % gnm_to_seq_dict[each_gnm])
    concatenated_msa_handle.close()

    # write out partition file
    end_pos = 0
    partition_file_handle = open(partition_file, 'w')
    for each_m in msa_file_list_sorted:
        gene_id = each_m.split('.' + msa_ext)[0]
        current_m_len = msa_len_dict[gene_id]
        partition_file_handle.write('%s = %s-%s\n' % (each_m, (end_pos + 1), (end_pos + current_m_len)))
        end_pos += current_m_len
    partition_file_handle.close()

    # convert msa in fasta to phy
    AlignIO.convert(concatenated_msa_fasta, 'fasta', concatenated_msa_phy, 'phylip-relaxed')


msa_dir                = '/Users/songweizhi/Desktop/s06_identified_marker_aln_trimmed'
msa_ext                = 'aln'
concatenated_msa_phy   = '/Users/songweizhi/Desktop/s06_identified_marker_aln_trimmed_concatenated.phy'
partition_file         = '/Users/songweizhi/Desktop/s06_identified_marker_aln_trimmed_concatenated_partition.txt'
# catfasta2phy(msa_dir, msa_ext, concatenated_msa_phy, partition_file)



msa_file = '/Users/songweizhi/Desktop/PA_75_DeltaLL_75_concatenated.phy'
msa_file_subset = '/Users/songweizhi/Desktop/PA_75_DeltaLL_75_concatenated_subset.phy'

from Bio import AlignIO

def slice_msa_by_col(msa_in, range_str, msa_out):
    alignment = AlignIO.read(msa_in, 'phylip-relaxed')

    range_l = int(range_str.split('-')[0]) - 1
    range_r = int(range_str.split('-')[1])

    aln_subset = alignment[:, range_l:range_r]
    AlignIO.write(aln_subset, msa_out, 'phylip-relaxed')


def slice_msa_by_col_manual(msa_in, range_str, msa_out):
    alignment = AlignIO.read(msa_in, 'phylip-relaxed')

    range_l = int(range_str.split('-')[0]) - 1
    range_r = int(range_str.split('-')[1])
    aln_subset = alignment[:, range_l:range_r]

    max_seq_id_len = 0
    for each_seq in aln_subset:
        seq_id_len = len(each_seq.id)
        if seq_id_len > max_seq_id_len:
            max_seq_id_len = seq_id_len
    print(max_seq_id_len)

    with open(msa_out, 'w') as msa_out_handle:
        msa_out_handle.write('%s %s\n' % (len(aln_subset), aln_subset.get_alignment_length()))
        for each_seq in aln_subset:
            seq_id = each_seq.id
            seq_id_with_space = '%s%s' % (seq_id, ' '*(max_seq_id_len + 2 - len(seq_id)))
            print(seq_id_with_space)
            msa_out_handle.write('%s%s\n' % (seq_id_with_space, str(each_seq.seq)))


#    AlignIO.write(aln_subset, msa_out, 'phylip-relaxed')


slice_range = ['1-500', '501-1000', '1001-1500', '1501-2000', '2001-2500', '2501-3000', '3001-3500', '3501-4000', '4001-4500', '4501-4879']

for each_range in slice_range:
    pwd_msa_op = '/Users/songweizhi/Desktop/%s.phy' % each_range
    slice_msa_by_col_manual(msa_file, each_range, pwd_msa_op)


def fa2phy(fasta_in, phy_out):
    alignment = AlignIO.read(fasta_in, 'fasta')
    max_seq_id_len = 0
    for each_seq in alignment:
        seq_id_len = len(each_seq.id)
        if seq_id_len > max_seq_id_len:
            max_seq_id_len = seq_id_len

    with open(phy_out, 'w') as msa_out_handle:
        msa_out_handle.write('%s %s\n' % (len(alignment), alignment.get_alignment_length()))
        for each_seq in alignment:
            seq_id = each_seq.id
            seq_id_with_space = '%s%s' % (seq_id, ' ' * (max_seq_id_len + 2 - len(seq_id)))
            msa_out_handle.write('%s%s\n' % (seq_id_with_space, str(each_seq.seq)))



import argparse
from ete3 import Tree


TaxonTree_usage = '''
================================ TaxonTree example commands ================================

TreeSAK TaxonTree -i ar53_r220.tree -tax o__Nitrososphaerales -o o__Nitrososphaerales.tree

============================================================================================
'''


def TaxonTree(args):

    tree_file_in     = args['i']
    interested_taxon = args['tax']
    tree_file_out    = args['o']

    input_tree = Tree(tree_file_in, quoted_node_names=True, format=1)

    matched_node_list = []
    for node in input_tree.traverse():
        if (node.name == interested_taxon) or (interested_taxon in node.name):
            matched_node_list.append(node.name)

    if len(matched_node_list) == 1:
        for node in input_tree.traverse():
            if node.name in matched_node_list:
                node.write(outfile=tree_file_out)
    else:
        print('There are multiple matched nodes. program exited!')
        print('Matched nodes: %s' % ','.join(matched_node_list))
        exit()

    print('Subset tree exported to: %s' % tree_file_out)
    print('Done!')


if __name__ == '__main__':

    TaxonTree_parser = argparse.ArgumentParser()
    TaxonTree_parser.add_argument('-i',   required=True,  help='input tree file')
    TaxonTree_parser.add_argument('-tax', required=True,  help='interested taxon')
    TaxonTree_parser.add_argument('-o',   required=True,  help='output tree file')
    args = vars(TaxonTree_parser.parse_args())
    TaxonTree(args)



import argparse
from Bio import AlignIO


fa2phy_usage = '''
======= fa2phy example commands =======

TreeSAK fa2phy -i msa.fa -o msa.phy

=======================================
'''


def fa2phy(args):

    fasta_in  = args['i']
    phy_out   = args['o']

    alignment = AlignIO.read(fasta_in, 'fasta')

    max_seq_id_len = 0
    for each_seq in alignment:
        seq_id_len = len(each_seq.id)
        if seq_id_len > max_seq_id_len:
            max_seq_id_len = seq_id_len

    with open(phy_out, 'w') as msa_out_handle:
        msa_out_handle.write('%s %s\n' % (len(alignment), alignment.get_alignment_length()))
        for each_seq in alignment:
            seq_id = each_seq.id
            seq_id_with_space = '%s%s' % (seq_id, ' ' * (max_seq_id_len + 2 - len(seq_id)))
            msa_out_handle.write('%s%s\n' % (seq_id_with_space, str(each_seq.seq)))


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-i',      required=True,   help='input MSA in fasta format')
    parser.add_argument('-o',      required=True,   help='output MSA in phylip format')
    args = vars(parser.parse_args())
    fa2phy(args)



import os
import argparse
from ete3 import Tree


mcmctree_vs_reltime_usage = '''
====================== mcmctree_vs_reltime example command ======================

TreeSAK mcmctree_vs_reltime -m mcmc.out -r reltime.txt -n nodes.txt -o ages.pdf

# Example data
https://github.com/songweizhi/TreeSAK/tree/master/DemoData/mcmctree_vs_reltime

=================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def get_lca(reltime_txt, leaf_1_name, leaf_2_name):

    leaf_set = set()
    child_to_parent_dict = dict()
    id_to_name_dict = dict()
    name_to_id_dict = dict()
    for each_line in open(reltime_txt):
        if not each_line.startswith('NodeLabel'):
            each_line_split = each_line.strip().split('\t')
            each_line_split = [i.strip() for i in each_line_split]
            if len(each_line_split) > 1:
                node_name = each_line_split[0].replace(' ', '_')
                node_id = each_line_split[1]
                des1 = each_line_split[2]
                des2 = each_line_split[3]
                id_to_name_dict[node_id] = node_name
                name_to_id_dict[node_name] = node_id
                child_to_parent_dict[des1] = node_id
                child_to_parent_dict[des2] = node_id
                if (des1 == '-') and (des2 == '-'):
                    leaf_set.add(node_id)

    leaf_to_lineage_dict = dict()
    for leaf in sorted([i for i in leaf_set]):
        original_leaf = leaf
        lineage_list = [leaf]
        while leaf in child_to_parent_dict:
            leaf_p = child_to_parent_dict[leaf]
            lineage_list.append(leaf_p)
            leaf = leaf_p
        leaf_to_lineage_dict[original_leaf] = lineage_list

    leaf_1_id     = name_to_id_dict[leaf_1_name]
    leaf_2_id     = name_to_id_dict[leaf_2_name]
    leaf_1_linage = leaf_to_lineage_dict[leaf_1_id]
    leaf_2_linage = leaf_to_lineage_dict[leaf_2_id]

    lca = ''
    for each_p in leaf_1_linage[::-1]:
        if each_p in leaf_2_linage:
            lca = each_p

    return lca


def parse_reltime(reltime_txt, interested_nodes_txt, op_txt):

    lca_to_leaves_dict = dict()
    interested_node_desc_dict = dict()
    for interested_node in open(interested_nodes_txt):
        interested_node_split = interested_node.strip().split('\t')
        paired_leaves = interested_node_split[0]
        interested_node_desc = paired_leaves
        if len(interested_node_split) > 1:
            interested_node_desc = interested_node_split[1]
        interested_node_desc_dict[paired_leaves] = interested_node_desc
        leaf_1 = paired_leaves.split(',')[0]
        leaf_2 = paired_leaves.split(',')[1]
        lca_id = get_lca(reltime_txt, leaf_1, leaf_2)
        lca_to_leaves_dict[lca_id] = paired_leaves.strip()

    op_txt_handle = open(op_txt, 'w')
    line_num_index = 0
    for each_line in open(reltime_txt):
        each_line_split = each_line.strip().split('\t')
        each_line_split = [i.strip() for i in each_line_split]
        if line_num_index == 0:
            op_txt_handle.write('ColorBy\tNode\tMean\tLow\tHigh\n')
        else:
            if len(each_line_split) > 1:
                node_id = each_line_split[1]
                if node_id in lca_to_leaves_dict:
                    node_id = each_line_split[1]
                    div_time = each_line_split[7]
                    ci_lower = each_line_split[8]
                    ci_upper = each_line_split[9]
                    corresponding_leaves = lca_to_leaves_dict[node_id]
                    interested_node_desc = interested_node_desc_dict[corresponding_leaves]
                    op_txt_handle.write('RelTime\t%s\t%s\t%s\t%s\n' % (interested_node_desc, div_time, ci_lower, ci_upper))
        line_num_index += 1
    op_txt_handle.close()


def mcmctree_out_to_tree_str(mamctree_out):

    # get tree string from mamctree_out
    tree_str = ''
    tree_line = 0
    current_line = 1
    for each_line in open(mamctree_out):
        if 'Species tree for FigTree.  Branch lengths = posterior mean times; 95% CIs = labels' in each_line:
            tree_line = current_line + 1
        if tree_line == current_line:
            tree_str = each_line.strip()
        current_line += 1

    tree_str_no_space = tree_str.replace(' ', '')

    # rename tree nodes
    t = Tree(tree_str_no_space, format=1)
    for each_node in t.traverse():
        if each_node.is_leaf():
            node_name_new = '_'.join(each_node.name.split('_')[1:])
        else:
            node_name_new = 't_n%s' % each_node.name
        each_node.name = node_name_new

    tree_str_renamed = t.write(format=8)

    return tree_str_renamed


def get_internal_node_to_plot(node_txt, mo_file):

    tree_str = ''
    if os.path.isfile(mo_file):
        tree_str = mcmctree_out_to_tree_str(mo_file)

    # get nodes to plot
    node_set = set()
    node_rename_dict = dict()
    if os.path.isfile(node_txt) is True:
        for each in open(node_txt):
            each_split = each.strip().split('\t')
            node_str = each_split[0]

            # get internal_node_to_plot
            internal_node_to_plot = ''
            if ',' not in node_str:
                internal_node_to_plot = each_split[0]
            else:
                leaf_list = node_str.split(',')
                if tree_str == '':
                    print('MCMCTree out file not found, program exited!')
                    exit()
                current_lca = Tree(tree_str, format=1).get_common_ancestor(leaf_list)
                internal_node_to_plot = current_lca.name

            # add internal_node_to_plot to  node_set
            if internal_node_to_plot != '':
                node_set.add(internal_node_to_plot)

            # read in name to show in plot
            if len(each_split) == 2:
                if each_split[1] != '':
                    node_rename_dict[internal_node_to_plot] = each_split[1]
    else:
        node_set = node_txt.split(',')

    return node_set, node_rename_dict, tree_str


def read_in_posterior_mean(mcmctree_out):

    # read in Posterior mean
    node_to_mean_hpd95_dict = dict()
    current_line = 1
    posterior_mean_header_line = 0
    for each_line in open(mcmctree_out):
        if 'Posterior mean (95% Equal-tail CI) (95% HPD CI) HPD-CI-width' in each_line:
            posterior_mean_header_line = current_line

        if (posterior_mean_header_line != 0) and (current_line > posterior_mean_header_line):
            each_line_split = each_line.strip().split(' ')

            each_line_split_no_empty = []
            for each_element in each_line_split:
                if each_element not in ['', '(']:
                    each_element_value = each_element.replace('(', '').replace(')', '').replace(',', '')
                    each_line_split_no_empty.append(each_element_value)
            if len(each_line_split_no_empty) == 9:
                node_id           = each_line_split_no_empty[0]
                value_mean        = each_line_split_no_empty[1]
                value_hpd95_small = each_line_split_no_empty[4]
                value_hpd95_big   = each_line_split_no_empty[5]
                node_to_mean_hpd95_dict[node_id] = [value_mean, value_hpd95_small, value_hpd95_big]
        current_line += 1

    return node_to_mean_hpd95_dict


def parse_mcmc_out(mcmc_out_file, node_txt, dm_out):

    dm_out_handle = open(dm_out, 'a')
    #dm_out_handle.write('Test\tShape\tVar\tMean\tLow\tHigh\n')
    node_set, node_rename_dict, tree_str = get_internal_node_to_plot(node_txt, mcmc_out_file)
    node_to_mean_95_hpd_dict = read_in_posterior_mean(mcmc_out_file)
    for each_node in node_set:
        node_name_to_write = node_rename_dict.get(each_node, each_node)
        mean_95_hpd_list = node_to_mean_95_hpd_dict.get(each_node)
        dm_out_handle.write('MCMCTree\t%s\t%s\n' % (node_name_to_write, '\t'.join(mean_95_hpd_list)))
    dm_out_handle.close()


def mcmctree_vs_reltime(args):

    mcmc_out_file           = args['m']
    reltime_txt             = args['r']
    interested_nodes_txt    = args['n']
    pdf_out                 = args['o']

    dm_out_combined = '.'.join(pdf_out.split('.')[:-1]) + '.txt'

    # define mcmctree_vs_reltime_R
    pwd_current_file  = os.path.realpath(__file__)
    current_file_path = '/'.join(pwd_current_file.split('/')[:-1])
    mcmctree_vs_reltime_R  = '%s/mcmctree_vs_reltime.R' % current_file_path

    parse_reltime(reltime_txt, interested_nodes_txt, dm_out_combined)
    parse_mcmc_out(mcmc_out_file, interested_nodes_txt, dm_out_combined)

    plot_cmd   = 'Rscript %s -i %s -x %s -y %s -o %s' % (mcmctree_vs_reltime_R, dm_out_combined, 8, 5, pdf_out)
    os.system(plot_cmd)
    print('Plot exported to: %s' % pdf_out)


if __name__ == '__main__':

    mcmctree_vs_reltime_parser = argparse.ArgumentParser()
    mcmctree_vs_reltime_parser.add_argument('-m',   required=True,  help='.out file from MCMCTree')
    mcmctree_vs_reltime_parser.add_argument('-r',   required=True,  help='output from elTime')
    mcmctree_vs_reltime_parser.add_argument('-n',   required=True,  help='interested nodes txt file')
    mcmctree_vs_reltime_parser.add_argument('-o',   required=True,  help='output pdf')
    args = vars(mcmctree_vs_reltime_parser.parse_args())
    mcmctree_vs_reltime(args)



import os
import glob
from Bio import SeqIO


def parse_deltall_stdout(deltall_stdout_txt, summary_txt):

    deltall_op_dict = dict()
    for each_line in open(deltall_stdout_txt):
        if not ((each_line.startswith('WARNING:')) or (each_line.startswith('awk:'))):
            each_line_split = each_line.strip().split('\t')
            marker_id = each_line_split[0]
            value = float(each_line_split[1])
            if marker_id not in deltall_op_dict:
                deltall_op_dict[marker_id] = [value]
            else:
                deltall_op_dict[marker_id].append(value)

    metric_1_dict = dict()
    metric_2_dict = dict()
    for each_marker in deltall_op_dict:
        metric_1_value = float("{0:.2f}".format(deltall_op_dict[each_marker][0]))
        metric_2_value = float("{0:.2f}".format(deltall_op_dict[each_marker][1]))
        metric_1_dict[each_marker] = metric_1_value
        metric_2_dict[each_marker] = metric_2_value

    metric_1_dict_sorted = {k: v for k, v in sorted(metric_1_dict.items(), key=lambda item: item[1])[::-1]}
    metric_2_dict_sorted = {k: v for k, v in sorted(metric_2_dict.items(), key=lambda item: item[1])}

    metric_1_score_dict = dict()
    metric_1_score = 1
    for each_marker_1 in metric_1_dict_sorted:
        metric_1_score_dict[each_marker_1] = metric_1_score
        metric_1_score += 1

    metric_2_score_dict = dict()
    metric_2_score = 1
    for each_marker_2 in metric_2_dict_sorted:
        metric_2_score_dict[each_marker_2] = metric_2_score
        metric_2_score += 1

    overall_score_dict = dict()
    for each_marker in deltall_op_dict:
        metric_score_1 = metric_1_score_dict[each_marker]
        metric_score_2 = metric_2_score_dict[each_marker]
        metric_score_overall = metric_score_1 + metric_score_2
        overall_score_dict[each_marker] = metric_score_overall

    overall_score_dict_sorted = {k: v for k, v in sorted(overall_score_dict.items(), key=lambda item: item[1])}

    summary_txt_handle = open(summary_txt, 'w')
    summary_txt_handle.write('Marker\tmetric1\tmetric1_score\tmetric2\tmetric2_score\toverall_score\n')
    for each_marker in overall_score_dict_sorted:
        metric_value_1 = metric_1_dict[each_marker]
        metric_value_2 = metric_2_dict[each_marker]
        metric_score_1 = metric_1_score_dict[each_marker]
        metric_score_2 = metric_2_score_dict[each_marker]
        metric_score_overall = overall_score_dict_sorted[each_marker]
        summary_txt_handle.write('%s\t%s\t%s\t%s\t%s\t%s\n' % (each_marker, metric_value_1, metric_score_1, metric_value_2, metric_score_2, metric_score_overall))
    summary_txt_handle.close()


def assess_markers_pa(trimmed_aln_dir, gnm_meta_txt, present_pct_cutoff_list, assess_summary_1_txt, assess_summary_2_txt):

    trimmed_aln_file_re = '%s/*.aln' % trimmed_aln_dir
    trimmed_aln_file_list = [os.path.basename(file_name) for file_name in glob.glob(trimmed_aln_file_re)]

    # read in genome metadata
    domain_to_gnm_dict = dict()
    gnm_to_domain_dict = dict()
    for each_gnm in open(gnm_meta_txt):
        each_gnm_split = each_gnm.strip().split('\t')
        gnm_id = each_gnm_split[0]
        domain_name = each_gnm_split[1]
        gnm_to_domain_dict[gnm_id] = domain_name

        if domain_name not in domain_to_gnm_dict:
            domain_to_gnm_dict[domain_name] = {gnm_id}
        else:
            domain_to_gnm_dict[domain_name].add(gnm_id)

    assess_summary_1_txt_handle = open(assess_summary_1_txt, 'w')
    assess_summary_2_txt_handle = open(assess_summary_2_txt, 'w')
    gnm_to_marker_dict = dict()
    marker_to_gnm_dict = dict()
    cutoff_to_qualified_marker_dict = dict()
    assess_summary_1_txt_handle.write('Marker\tArchaea\tEukaryota\n')
    assess_summary_2_txt_handle.write('Marker\t%s\n' % '\t'.join([str(i) for i in present_pct_cutoff_list]))
    for each_aln in trimmed_aln_file_list:
        marker_id = each_aln.split('.aln')[0]
        pwd_aln = '%s/%s' % (trimmed_aln_dir, each_aln)
        gnm_set = set()
        for each_seq in SeqIO.parse(pwd_aln, 'fasta'):
            gnm_id = each_seq.id
            gnm_set.add(gnm_id)
            if gnm_id not in gnm_to_marker_dict:
                gnm_to_marker_dict[gnm_id] = {marker_id}
            else:
                gnm_to_marker_dict[gnm_id].add(marker_id)
        marker_to_gnm_dict[marker_id] = gnm_set

        gnm_num_ar = 0
        gnm_num_eu = 0
        for each_g in gnm_set:
            g_domain = gnm_to_domain_dict[each_g]
            if g_domain == 'Archaea':
                gnm_num_ar += 1
            if g_domain == 'Eukaryota':
                gnm_num_eu += 1
        gnm_pct_ar = float("{0:.2f}".format(gnm_num_ar / 133 * 100))
        gnm_pct_eu = float("{0:.2f}".format(gnm_num_eu / 27 * 100))

        # assessment
        assessment_result_list = []
        for present_pct_cutoff in present_pct_cutoff_list:
            if (gnm_pct_ar >= present_pct_cutoff) and (gnm_pct_eu >= present_pct_cutoff):
                assessment_result_list.append('1')
                if str(present_pct_cutoff) not in cutoff_to_qualified_marker_dict:
                    cutoff_to_qualified_marker_dict[str(present_pct_cutoff)] = [marker_id]
                else:
                    cutoff_to_qualified_marker_dict[str(present_pct_cutoff)].append(marker_id)
            else:
                assessment_result_list.append('0')
        assess_summary_1_txt_handle.write('%s\t%s\t%s\n' % (marker_id, gnm_pct_ar, gnm_pct_eu))
        assess_summary_2_txt_handle.write('%s\t%s\n' % (marker_id, '\t'.join(assessment_result_list)))

    summary_list = [len(cutoff_to_qualified_marker_dict.get(str(i), [])) for i in present_pct_cutoff_list]
    summary_list_str = [str(j) for j in summary_list]
    assess_summary_2_txt_handle.write('Total\t%s\n' % ('\t'.join(summary_list_str)))
    assess_summary_1_txt_handle.close()
    assess_summary_2_txt_handle.close()

    return cutoff_to_qualified_marker_dict, gnm_to_marker_dict, marker_to_gnm_dict


def read_in_assessment_pa_2_txt(assessment_pa_2_txt):

    pa_pct_list = []
    pa_pct_to_marker_dict = dict()
    for each_marker in open(assessment_pa_2_txt):
        each_marker_split = each_marker.strip().split('\t')
        if each_marker.startswith('Marker\t'):
            pa_pct_list = [int(i) for i in each_marker_split[1:]]

            # initialize pa_pct_to_marker_dict
            for pa_pct in pa_pct_list:
                pa_pct_to_marker_dict[pa_pct] = set()

        elif not each_marker.startswith('Total\t'):
            marker_id = each_marker_split[0]
            for (pct, pa) in zip(pa_pct_list, each_marker_split[1:]):
                if pa == '1':
                    pa_pct_to_marker_dict[pct].add(marker_id)

    return pa_pct_to_marker_dict


def get_marker_set_dict(assessment_pa_2_txt, assessment_deltall_txt, deltall_keep_pct_list, min_marker_num):

    # read in assessment_pa_2_txt
    pa_pct_to_marker_dict = read_in_assessment_pa_2_txt(assessment_pa_2_txt)
    # for each in pa_pct_to_marker_dict:
    #     print('%s(%s)\t%s' % (each, len(pa_pct_to_marker_dict[each]), pa_pct_to_marker_dict[each]))

    # store marker in list according to their DeltaLl scores
    marker_list_by_score = []
    for each_marker in open(assessment_deltall_txt):
        if not each_marker.startswith('Marker\t'):
            each_marker_split = each_marker.strip().split('\t')
            marker_id = each_marker_split[0]
            marker_list_by_score.append(marker_id)

    # get intersections
    marker_set_dict = dict()
    for each_keep_pct in deltall_keep_pct_list:
        keep_num = round(len(marker_list_by_score) * each_keep_pct / 100)
        if keep_num >= min_marker_num:
            marker_to_keep = marker_list_by_score[:keep_num]
            # print('deltall_keep_pct(top %s%s)(%s)\t%s' % (each_keep_pct, '%', keep_num, marker_to_keep))
            for each_pa_pct in pa_pct_to_marker_dict:
                current_pa_pct_marker_set = pa_pct_to_marker_dict[each_pa_pct]
                marker_set_key = 'deltall%s_pa%s' % (each_keep_pct, each_pa_pct)
                marker_shared = set(marker_to_keep).intersection(current_pa_pct_marker_set)
                marker_set_dict[marker_set_key] = marker_shared

    return marker_set_dict


def AssessMarker(assess_marker_wd, trimmed_aln_dir, gnm_group_txt, deltall_stdout_txt, present_pct_cutoff_list, deltall_keep_pct_list, min_marker_pct_per_gnm, min_marker_num, force_create_dir, catfasta2phyml_pl):

    # define output file name
    assess_summary_deltall_txt     = '%s/assessment_deltall.txt'      % assess_marker_wd
    assess_summary_1_txt_by_marker = '%s/assessment_pa_1.txt'         % assess_marker_wd
    assess_summary_2_txt_by_marker = '%s/assessment_pa_2.txt'         % assess_marker_wd
    assess_summary_txt_by_genome   = '%s/assessment_pa_by_genome.txt' % assess_marker_wd

    # parse deltall stdout
    parse_deltall_stdout(deltall_stdout_txt, assess_summary_deltall_txt)

    # assess markers
    cutoff_to_qualified_marker_dict, gnm_to_marker_dict, marker_to_gnm_dict = assess_markers_pa(trimmed_aln_dir, gnm_group_txt, present_pct_cutoff_list, assess_summary_1_txt_by_marker, assess_summary_2_txt_by_marker)

    # write out qualified markers
    for each_cutoff in cutoff_to_qualified_marker_dict:
        qualified_m_list = sorted(cutoff_to_qualified_marker_dict[each_cutoff])
        pwd_op_txt = '%s/assessment_pa_qualified_marker_%s.txt' % (assess_marker_wd, each_cutoff)
        with open(pwd_op_txt, 'w') as pwd_op_txt_handle:
            pwd_op_txt_handle.write('\n'.join(qualified_m_list))

    # write out summary by genomes
    assess_summary_txt_by_genome_handle = open(assess_summary_txt_by_genome, 'w')
    assess_summary_txt_by_genome_handle.write('Cutoff\tGenome\tMarker_all\tMarker_qualified\tMarker_qualified_pct(cutoff:%s)\n' % min_marker_pct_per_gnm)
    for each_cutoff in present_pct_cutoff_list:
        current_cutoff_qualified_marker_list = cutoff_to_qualified_marker_dict.get(str(each_cutoff), [])
        if len(current_cutoff_qualified_marker_list) > 0:
            qualified_gnm_set = set()
            for each_gnm in gnm_to_marker_dict:
                gnm_identified_marker_set = gnm_to_marker_dict[each_gnm]
                gnm_identified_marker_set_qualified = set()
                for identified_marker in gnm_identified_marker_set:
                    if identified_marker in current_cutoff_qualified_marker_list:
                        gnm_identified_marker_set_qualified.add(identified_marker)
                gnm_identified_marker_set_qualified_pct = len(gnm_identified_marker_set_qualified)*100/len(current_cutoff_qualified_marker_list)
                gnm_identified_marker_set_qualified_pct = float("{0:.2f}".format(gnm_identified_marker_set_qualified_pct))
                if gnm_identified_marker_set_qualified_pct >= min_marker_pct_per_gnm:
                    qualified_gnm_set.add(each_gnm)
                else:
                    assess_summary_txt_by_genome_handle.write('%s\t%s\t%s\t%s\t%s\n' % (each_cutoff, each_gnm, len(gnm_identified_marker_set), len(gnm_identified_marker_set_qualified), gnm_identified_marker_set_qualified_pct))
        assess_summary_txt_by_genome_handle.write('\n')
    assess_summary_txt_by_genome_handle.close()

    # select marker gene and concatenate the alignments
    marker_set_dict = get_marker_set_dict(assess_summary_2_txt_by_marker, assess_summary_deltall_txt, deltall_keep_pct_list, min_marker_num)
    for each_marker_set in marker_set_dict:
        current_marker_set = marker_set_dict[each_marker_set]
        if len(current_marker_set) >= min_marker_num:

            #print('%s\t%s\t%s' % (each_marker_set, len(current_marker_set), current_marker_set))
            pwd_iqtree_dir    = '%s/%s_iqtree_wd'       % (assess_marker_wd, each_marker_set)
            pwd_marker_id_txt = '%s/%s_marker_id.txt'   % (pwd_iqtree_dir, each_marker_set)
            pwd_aln_dir       = '%s/%s_aln_trimmed'     % (pwd_iqtree_dir, each_marker_set)

            # copy marker alignments into corresponding dir
            if force_create_dir is True:
                if os.path.isdir(pwd_iqtree_dir) is True:
                    os.system('rm -r %s' % pwd_iqtree_dir)
            os.system('mkdir %s' % pwd_iqtree_dir)
            os.system('mkdir %s' % pwd_aln_dir)

            # write out marker id
            with open(pwd_marker_id_txt, 'w') as pwd_marker_id_txt_handle:
                pwd_marker_id_txt_handle.write('\n'.join(current_marker_set))

            for each_aln in current_marker_set:
                pwd_aln = '%s/%s.aln' % (trimmed_aln_dir, each_aln)
                cp_cmd = 'cp %s %s/' % (pwd_aln, pwd_aln_dir)
                os.system(cp_cmd)

            # concatenate alignment
            pwd_concatenate_aln            = '%s/%s_concatenated.phy'                                 % (pwd_iqtree_dir, each_marker_set)
            pwd_concatenate_aln_partitions = '%s/%s_partitions.txt'                                   % (pwd_iqtree_dir, each_marker_set)
            catfasta2phyml_cmd             = 'perl %s --sequential --concatenate %s/*.aln > %s 2> %s' % (catfasta2phyml_pl, pwd_aln_dir, pwd_concatenate_aln, pwd_concatenate_aln_partitions)
            #print(catfasta2phyml_cmd)
            os.system(catfasta2phyml_cmd)

            # get guide tree
            get_guide_cmd = 'iqtree -s %s_concatenated.phy --prefix %s_guide_tree --seqtype AA -m LG -T 12 -B 1000 --alrt 1000' % (each_marker_set, each_marker_set)
            # print(get_guide_cmd)

            # run C60 + PMSF
            c60_pmsf_cmd = 'iqtree -s %s_concatenated.phy --prefix %s --seqtype AA -m LG+G+F+C60 -T 12 -B 1000 --alrt 1000 -ft %s_guide_tree.treefile' % (each_marker_set, each_marker_set, each_marker_set)
            # print(c60_pmsf_cmd)

            # generate job script
            pwd_js = '%s/js_%s_iqtree.sh' % (assess_marker_wd, each_marker_set)
            with open(pwd_js, 'w') as pwd_js_handle:
                #pwd_js_handle.write('#!/bin/bash\n#SBATCH --nodelist cl007\n#SBATCH --ntasks 1\n#SBATCH --cpus-per-task 12\n\n')
                pwd_js_handle.write('#!/bin/bash\n#SBATCH\n#SBATCH --ntasks 1\n#SBATCH --cpus-per-task 12\n\n')
                pwd_js_handle.write('cd %s_iqtree_wd\n' % each_marker_set)
                pwd_js_handle.write(get_guide_cmd + '\n')
                pwd_js_handle.write(c60_pmsf_cmd + '\n')


# inputs
assess_marker_wd        = '/home-user/wzsong/DateArTree/04_dating_Williams_2017_45_arCOG_assess_marker'
trimmed_aln_dir         = '/home-user/wzsong/DateArTree/02_identify_marker_gene_Williams_2017_45_arCOG/best_hit_by_marker_5_aln_trimmed'
gnm_group_txt           = '/home-user/wzsong/DateArTree/01_genome_selection/gnm_metadata.txt'
deltall_stdout_txt      = '/home-user/wzsong/DateArTree/02_identify_marker_gene_Williams_2017_45_arCOG_DeltaLL/nohup.out'
present_pct_cutoff_list = [25, 50, 75, 85, 100]
deltall_keep_pct_list   = [25, 50, 75, 100]
min_marker_pct_per_gnm  = 75
min_marker_num          = 20
force_create_dir        = True
#catfasta2phyml_pl       = '/Users/songweizhi/PycharmProjects/Sponge_Hologenome/Scripts/catfasta2phyml.pl'
catfasta2phyml_pl       = '/home-user/wzsong/Scripts/catfasta2phyml.pl'

AssessMarker(assess_marker_wd, trimmed_aln_dir, gnm_group_txt, deltall_stdout_txt, present_pct_cutoff_list, deltall_keep_pct_list, min_marker_pct_per_gnm, min_marker_num, force_create_dir, catfasta2phyml_pl)


'''
Note
1. Extra genomes in gnm_metadata.txt won't affect assessment results.
2. Genomes can not be found in gnm_metadata.txt will trigger an error.
3. Alignments in {trimmed_aln_dir} need to be trimmed before assessment
4. Sequences in MSAs need to be named by genome id.
'''



import os
import argparse
from distutils.spawn import find_executable


PMSF_usage = '''
==================== PMSF example commands ====================

# Dependency: iqtree2

TreeSAK PMSF -i in.aln -o get_PMSF_tree_wd -t 12
TreeSAK PMSF -i in.aln -o get_PMSF_tree_wd -t 12 -topo topo.tree
TreeSAK PMSF -i in.aln -o get_PMSF_tree_wd -t 12 -m C60SR4 -mdef C60SR4.nex

# This is a wrapper for:
iqtree2 -T 12 -B 1000 --alrt 1000 --quiet --seqtype AA -s in.aln --prefix guide_tree -m LG+F+G 
iqtree2 -T 12 -B 1000 --alrt 1000 --quiet --seqtype AA -s in.aln --prefix PMSF -m LG+C60+F+G -ft guide_tree.treefile

# more information: http://www.iqtree.org/doc/Complex-Models

# Reference: The evolutionary origin of host association in the Rickettsiales
Maximum likelihood phylogenetic reconstructions were done under the PMSF approximation 
(with 100 non-parametric bootstraps; guidetree under LG+G+F) of the LG+C60+F+Γ4 model 
(selected by ModelFinder) for both supermatrix alignments with IQTREE v1.6.5.

===============================================================
'''


def PMSF(args):

    msa_in                  = args['i']
    iqtree_model_guide_tree = args['gm']
    iqtree_model            = args['m']
    mdef_nex                = args['mdef']
    op_dir                  = args['o']
    tree_prefix             = args['p']
    force_overwrite         = args['f']
    num_of_threads          = args['t']
    topo_constraint_txt     = args['topo']

    guide_tree_wd  = '%s/guide_tree'           % op_dir
    pwd_guide_tree = '%s/guide_tree.treefile'  % guide_tree_wd
    pwd_cmd_txt    = '%s/cmds.txt'             % op_dir

    iqtree_exe = ''
    if find_executable('iqtree2'):
        iqtree_exe = 'iqtree2'
    elif find_executable('iqtree'):
        iqtree_exe = 'iqtree'
    else:
        print('iqtree not detected, program exited!')
        exit()

    # check input file
    if os.path.isfile(msa_in) is False:
        print('MSA file not found, program exited!')
        exit()

    # create output dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('output folder already exist, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % guide_tree_wd)

    guidetree_cmd = '%s -s %s --prefix %s/guide_tree --seqtype AA -m %s -T %s -B 1000 --alrt 1000 --quiet' % (iqtree_exe, msa_in, guide_tree_wd, iqtree_model_guide_tree, num_of_threads)
    iqtree_cmd    = '%s -s %s --prefix %s/%s --seqtype AA -m %s -T %s -B 1000 --alrt 1000 --quiet -ft %s'  % (iqtree_exe, msa_in, op_dir, tree_prefix, iqtree_model, num_of_threads, pwd_guide_tree)
    if topo_constraint_txt is not None:
        guidetree_cmd = '%s -s %s --prefix %s/guide_tree --seqtype AA -m %s -T %s -B 1000 --alrt 1000 --quiet -g %s' % (iqtree_exe, msa_in, guide_tree_wd, iqtree_model_guide_tree, num_of_threads, topo_constraint_txt)
        iqtree_cmd    = '%s -s %s --prefix %s/%s --seqtype AA -m %s -T %s -B 1000 --alrt 1000 --quiet -ft %s -g %s'  % (iqtree_exe, msa_in, op_dir, tree_prefix, iqtree_model, num_of_threads, pwd_guide_tree, topo_constraint_txt)

    if mdef_nex is not None:
        iqtree_cmd = '%s --mdef %s' % (iqtree_cmd, mdef_nex)

    if iqtree_model == 'C60SR4':
        iqtree_cmd = iqtree_cmd.replace('--seqtype AA ', '')

    # write out commands
    pwd_cmd_txt_handle = open(pwd_cmd_txt, 'w')
    pwd_cmd_txt_handle.write(guidetree_cmd + '\n')
    pwd_cmd_txt_handle.write(iqtree_cmd + '\n')
    pwd_cmd_txt_handle.close()

    # get guide tree
    print('Building guide tree')
    print(guidetree_cmd)
    os.system(guidetree_cmd)

    # get PMSF tree
    print('Building PMSF tree with model %s' % iqtree_model)
    print(iqtree_cmd)
    os.system(iqtree_cmd)

    print('Done!')


if __name__ == '__main__':

    # initialize the options parser
    PMSF_parser = argparse.ArgumentParser()
    PMSF_parser.add_argument('-i',       required=True,                          help='input MSA file')
    PMSF_parser.add_argument('-gm',      required=False, default='LG+F+G',       help='iqtree model for guide tree, default: LG+F+G')
    PMSF_parser.add_argument('-m',       required=False, default='LG+C60+F+G',   help='iqtree model, default: LG+C60+F+G')
    PMSF_parser.add_argument('-mdef',    required=False, default=None,           help='model definition NEXUS file')
    PMSF_parser.add_argument('-o',       required=True,                          help='output plot')
    PMSF_parser.add_argument('-p',       required=False, default='PMSF',         help='tree prefix, default: PMSF')
    PMSF_parser.add_argument('-topo',    required=False, default=None,           help='topological constraint tree, pass to -g, default is None')
    PMSF_parser.add_argument('-t',       required=False, type=int, default=1,    help='num of threads')
    PMSF_parser.add_argument('-f',       required=False, action="store_true",    help='force overwrite')
    args = vars(PMSF_parser.parse_args())
    PMSF(args)



import os
import glob
import argparse
from Bio import SeqIO
from Bio import AlignIO
import multiprocessing as mp
from distutils.spawn import find_executable


MarkerRef2Tree_usage = '''
============================= MarkerRef2Tree example commands =============================

Dependencies: java, blastp, mafft-einsi, trimal, iqtree2

# example commands
TreeSAK MarkerRef2Tree -i faa_files -x faa -m marker_seq -mx fa -o output_dir -bmge -e 10 -t 6 -c 85
TreeSAK MarkerRef2Tree -i faa_files -x faa -m marker_seq -mx fa -o output_dir -bmge -e 10 -t 6 -c 75,100

# file extension need to be faa

===========================================================================================
'''


def check_dependencies(program_list):

    # check whether executables exist
    not_detected_programs = []
    for needed_program in program_list:
        if find_executable(needed_program) is None:
            not_detected_programs.append(needed_program)

    # report
    if not_detected_programs != []:
        print('%s not found, program exited!' % ','.join(not_detected_programs))
        exit()


def exe_cmds(cmd_list, num_threads):
    print('Running %s commands with %s cores' % (len(cmd_list), num_threads))
    pool = mp.Pool(processes=num_threads)
    pool.map(os.system, cmd_list)
    pool.close()
    pool.join()


def sep_path_basename_ext(file_in):
    file_path, file_name = os.path.split(file_in)
    if file_path == '':
        file_path = '.'
    file_basename, file_extension = os.path.splitext(file_name)
    return file_path, file_basename, file_extension


def catfasta2phy(msa_dir, msa_ext, concatenated_msa_phy, concatenated_msa_fasta, partition_file):

    msa_file_re            = '%s/*.%s'  % (msa_dir, msa_ext)
    msa_file_list          = [os.path.basename(file_name) for file_name in glob.glob(msa_file_re)]
    msa_file_list_sorted   = sorted(msa_file_list)

    complete_gnm_set = set()
    for each_msa_file in msa_file_list:
        pwd_msa = '%s/%s' % (msa_dir, each_msa_file)
        for each_seq in SeqIO.parse(pwd_msa, 'fasta'):
            complete_gnm_set.add(each_seq.id)

    complete_gnm_list_sorted = sorted([i for i in complete_gnm_set])

    # initialize concatenated msa dict
    gnm_to_seq_dict = {i: '' for i in complete_gnm_list_sorted}
    msa_len_dict = dict()
    for each_msa_file in msa_file_list_sorted:
        gene_id = each_msa_file.split('.' + msa_ext)[0]

        # read in msa
        current_msa_len = 0
        current_msa_len_set = set()
        pwd_current_msa = '%s/%s' % (msa_dir, each_msa_file)
        current_msa_seq_dict = dict()
        for each_seq in SeqIO.parse(pwd_current_msa, 'fasta'):
            complete_gnm_set.add(each_seq.id)
            current_msa_seq_dict[each_seq.id] = str(each_seq.seq)
            current_msa_len_set.add(len(each_seq.seq))
            current_msa_len = len(each_seq.seq)

        if len(current_msa_len_set) != 1:
            print('Sequences with different length were found in %s, program exited!' % each_msa_file)
            exit()

        msa_len_dict[gene_id] = current_msa_len

        # add sequence to concatenated msa dict
        for each_gnm in complete_gnm_list_sorted:
            msa_seq = current_msa_seq_dict.get(each_gnm, current_msa_len*'-')
            gnm_to_seq_dict[each_gnm] += msa_seq

    # write out concatenated msa
    concatenated_msa_handle = open(concatenated_msa_fasta, 'w')
    for each_gnm in complete_gnm_list_sorted:
        concatenated_msa_handle.write('>%s\n' % each_gnm)
        concatenated_msa_handle.write('%s\n' % gnm_to_seq_dict[each_gnm])
    concatenated_msa_handle.close()

    # write out partition file
    end_pos = 0
    partition_file_handle = open(partition_file, 'w')
    for each_m in msa_file_list_sorted:
        gene_id = each_m.split('.' + msa_ext)[0]
        current_m_len = msa_len_dict[gene_id]
        partition_file_handle.write('%s = %s-%s\n' % (each_m, (end_pos + 1), (end_pos + current_m_len)))
        end_pos += current_m_len
    partition_file_handle.close()

    # convert fasta to phy
    AlignIO.convert(concatenated_msa_fasta, 'fasta', concatenated_msa_phy, 'phylip-relaxed')


def select_seq(seq_file, id_file,select_option, output_file, one_line, in_fastq):

    # get provided id list
    seq_id_list = set()
    for seq_id in open(id_file):
        seq_id_list.add(seq_id.strip())

    seq_in_format = 'fasta'
    if in_fastq is True:
        seq_in_format = 'fastq'

    # extract sequences
    output_file_handle = open(output_file, 'w')
    for seq_record in SeqIO.parse(seq_file, seq_in_format):
        seq_id = seq_record.id
        if select_option == 1:
            if seq_id in seq_id_list:

                if in_fastq is False:
                    if one_line is False:
                        SeqIO.write(seq_record, output_file_handle, 'fasta')
                    else:
                        SeqIO.write(seq_record, output_file_handle, 'fasta-2line')
                else:
                    SeqIO.write(seq_record, output_file_handle, 'fastq')

        if select_option == 0:
            if seq_id not in seq_id_list:

                if in_fastq is False:
                    if one_line is False:
                        SeqIO.write(seq_record, output_file_handle, 'fasta')
                    else:
                        SeqIO.write(seq_record, output_file_handle, 'fasta-2line')
                else:
                    SeqIO.write(seq_record, output_file_handle, 'fastq')
    output_file_handle.close()


def AssessMarkerPA(trimmed_aln_dir, gnm_set, group_to_gnm_dict, present_pct_cutoff_list, op_dir):

    group_to_gnm_num_dict = dict()
    gnm_to_group_dict     = dict()
    for each_g in group_to_gnm_dict:
        gnm_member_list = group_to_gnm_dict[each_g]
        group_to_gnm_num_dict[each_g] = len(gnm_member_list)
        for each_gnm in gnm_member_list:
            gnm_to_group_dict[each_gnm] = each_g
    group_id_list_sorted = sorted(list(group_to_gnm_dict.keys()))

    # exit program if group information is missing
    gnms_without_group_info = set()
    for gnm in gnm_set:
        if gnm not in gnm_to_group_dict:
            gnms_without_group_info.add(gnm)

    if len(gnms_without_group_info) > 0:
        print('Group information for the following genomes are missing, program exited!')
        print(','.join(gnms_without_group_info))
        print('Group information for the above genomes are missing, program exited!')
        exit()

    # read in provided cutoffs
    assess_summary_1_txt    = '%s/assess_by_PA.txt'                % op_dir
    assess_summary_2_txt    = '%s/assess_by_PA_summary.txt'        % op_dir
    itol_binary_txt         = '%s/assess_by_PA_iTOL_binary.txt'    % op_dir

    trimmed_aln_file_re = '%s/*.aln' % (trimmed_aln_dir)
    trimmed_aln_file_list = [os.path.basename(file_name) for file_name in glob.glob(trimmed_aln_file_re)]

    assess_summary_1_txt_handle = open(assess_summary_1_txt, 'w')
    assess_summary_1_txt_handle.write('Marker\t%s\n' % '\t'.join([str(i) for i in group_id_list_sorted]))
    assess_summary_2_txt_handle = open(assess_summary_2_txt, 'w')
    assess_summary_2_txt_handle.write('Marker\t%s\n' % '\t'.join([str(i) for i in present_pct_cutoff_list]))
    cutoff_to_qualified_marker_dict = dict()
    gnm_to_identified_marker_dict = dict()
    marker_id_list = []
    for each_aln in trimmed_aln_file_list:

        marker_id = each_aln.split(('.aln'))[0]
        marker_id_list.append(marker_id)
        pwd_aln = '%s/%s' % (trimmed_aln_dir, each_aln)

        current_marker_num_by_group_dict = dict()
        for each_seq in SeqIO.parse(pwd_aln, 'fasta'):
            gnm_id = each_seq.id

            # get genome to marker dist
            if gnm_id not in gnm_to_identified_marker_dict:
                gnm_to_identified_marker_dict[gnm_id] = {marker_id}
            else:
                gnm_to_identified_marker_dict[gnm_id].add(marker_id)

            if gnm_id in gnm_to_group_dict:
                gnm_group = gnm_to_group_dict[gnm_id]
                if gnm_group not in current_marker_num_by_group_dict:
                    current_marker_num_by_group_dict[gnm_group] = 1
                else:
                    current_marker_num_by_group_dict[gnm_group] += 1
            else:
                print('Not all genomes used to generate the MSA being found in -aa, program exited!')
                exit()

        # write out assess_summary_1_txt
        pct_list = []
        for each_grp in group_id_list_sorted:
            grp_pct = current_marker_num_by_group_dict.get(each_grp, 0)*100/group_to_gnm_num_dict[each_grp]
            grp_pct = float("{0:.2f}".format(grp_pct))
            pct_list.append(grp_pct)
        assess_summary_1_txt_handle.write('%s\t%s\n' % (marker_id, '\t'.join([str(i) for i in pct_list])))

        # write out assess_summary_2_txt
        assess_list = []
        for each_cutoff in present_pct_cutoff_list:

            good_marker = True
            for each_pct in pct_list:
                if each_pct < each_cutoff:
                    good_marker = False

            if each_cutoff not in cutoff_to_qualified_marker_dict:
                cutoff_to_qualified_marker_dict[each_cutoff] = {marker_id}

            if good_marker is True:
                assess_list.append('1')
                cutoff_to_qualified_marker_dict[each_cutoff].add(marker_id)
            else:
                assess_list.append('0')
        assess_summary_2_txt_handle.write('%s\t%s\n' % (marker_id, '\t'.join(assess_list)))

    # write out total in assess_summary_2_txt
    total_stats_list = [str(len(cutoff_to_qualified_marker_dict[each_c])) for each_c in present_pct_cutoff_list]
    assess_summary_2_txt_handle.write('Total\t%s\n' % ('\t'.join(total_stats_list)))
    assess_summary_1_txt_handle.close()
    assess_summary_2_txt_handle.close()

    # copy alignments of qualified marker to corresponding folders
    for each_cutoff in cutoff_to_qualified_marker_dict:
        qualified_marker_set        = cutoff_to_qualified_marker_dict[each_cutoff]
        pwd_qualified_marker_dir    = '%s/marker_PA%s'        % (op_dir, each_cutoff)
        pwd_qualified_marker_id_txt = '%s/marker_PA%s_id.txt' % (op_dir, each_cutoff)

        os.system('mkdir %s' % pwd_qualified_marker_dir)
        for each_marker in qualified_marker_set:
            pwd_marker_aln = '%s/%s.aln' % (trimmed_aln_dir, each_marker)
            cp_cmd = 'cp %s %s/' % (pwd_marker_aln, pwd_qualified_marker_dir)
            os.system(cp_cmd)

        # write out id
        with open(pwd_qualified_marker_id_txt, 'w') as pwd_qualified_marker_id_txt_handle:
            pwd_qualified_marker_id_txt_handle.write('%s\n' % '\n'.join(qualified_marker_set))

    # write out iTOL file
    itol_binary_txt_handle = open(itol_binary_txt, 'w')
    itol_binary_txt_handle.write('DATASET_BINARY\n\nSEPARATOR TAB\nDATASET_LABEL\tlabel1\nCOLOR\t#85C1E9\n')
    itol_binary_txt_handle.write('SHOW_LABELS\t1\nLABEL_ROTATION\t45\nLABEL_SHIFT\t5\n')
    itol_binary_txt_handle.write('FIELD_LABELS\t%s\n' % '\t'.join(sorted(marker_id_list)))
    itol_binary_txt_handle.write('FIELD_SHAPES\t%s\n' % '\t'.join(['1']*len(marker_id_list)))
    itol_binary_txt_handle.write('\nDATA\n')
    for each_g in gnm_to_identified_marker_dict:
        g_identified_marker_set = gnm_to_identified_marker_dict[each_g]

        pa_list = []
        for each_m in sorted(marker_id_list):
            if each_m in g_identified_marker_set:
                pa_list.append('1')
            else:
                pa_list.append('-1')
        itol_binary_txt_handle.write('%s\t%s\n' % (each_g, '\t'.join(pa_list)))
    itol_binary_txt_handle.close()

    print('Assessment results exported to:\n%s\n%s' % (assess_summary_1_txt, assess_summary_2_txt))


def get_gap_stats(msa_in_fa, stats_txt):

    gap_pct_dict = dict()
    for each_seq in SeqIO.parse(msa_in_fa, 'fasta'):
        seq_id = each_seq.id
        seq_str = str(each_seq.seq)
        gap_pct = seq_str.count('-')*100/len(seq_str)
        gap_pct = float("{0:.2f}".format(gap_pct))
        gap_pct_dict[seq_id] = gap_pct

    gap_pct_sorted = sorted(gap_pct_dict.items(), key=lambda x:x[1])

    stats_txt_handle = open(stats_txt, 'w')
    stats_txt_handle.write('Sequence\tGap\n')
    for each_seq in gap_pct_sorted:
        stats_txt_handle.write('%s\t%s\n' % (each_seq[0], each_seq[1]))
    stats_txt_handle.close()


def BMGE(msa_in, op_prefix, trim_model, entropy_score_cutoff):

    # define file name
    msa_out_phylip = '%s.BMGE.phylip' % op_prefix
    msa_out_fasta  = '%s.BMGE.fasta'  % op_prefix
    msa_out_nexus  = '%s.BMGE.nexus'  % op_prefix
    msa_out_html   = '%s.BMGE.html'   % op_prefix

    # specify path to BMGE.jar
    current_file_path   = '/'.join(os.path.realpath(__file__).split('/')[:-1])
    pwd_bmge_jar        = '%s/BMGE.jar' % current_file_path

    # run BMGE
    bmge_cmd = 'java -jar %s -i %s -m %s -t AA -h %s -op %s -of %s -on %s -oh %s' % (pwd_bmge_jar, msa_in, trim_model, entropy_score_cutoff, msa_out_phylip, msa_out_fasta, msa_out_nexus, msa_out_html)
    print('Running %s' % bmge_cmd)
    os.system(bmge_cmd)


def MarkerRef2Tree(args):

    faa_file_dir                = args['i']
    faa_file_ext                = args['x']
    marker_seq_dir              = args['m']
    marker_seq_ext              = args['mx']
    gnm_group_txt               = args['g']
    op_dir                      = args['o']
    e_value                     = args['e']
    num_of_threads              = args['t']
    force_overwrite             = args['f']
    pa_cutoff_str               = args['c']
    minimal_marker_number       = args['mmn']
    run_psiblast                = args['psiblast']
    trim_with_bmge              = args['bmge']
    bmge_trim_model             = args['bmge_m']
    bmge_entropy_score_cutoff   = args['bmge_esc']

    # check dependencies
    check_dependencies(['java', 'psiblast', 'blastp', 'mafft-einsi', 'trimal', 'iqtree'])

    # specify path to BMGE.jar
    current_file_path = '/'.join(os.path.realpath(__file__).split('/')[:-1])
    pwd_bmge_jar      = '%s/BMGE.jar' % current_file_path

    # get marker id set
    marker_seq_re   = '%s/*.%s' % (marker_seq_dir, marker_seq_ext)
    marker_seq_list = [os.path.basename(file_name) for file_name in glob.glob(marker_seq_re)]
    marker_id_set = set()
    for each_marker_seq_file in marker_seq_list:
        _, marker_seq_basename, _ = sep_path_basename_ext(each_marker_seq_file)
        marker_id_set.add(marker_seq_basename)

    # get gnm id list
    faa_file_re   = '%s/*.%s' % (faa_file_dir, faa_file_ext)
    faa_file_list = [os.path.basename(file_name) for file_name in glob.glob(faa_file_re)]
    gnm_set = set()
    for each_faa_file in faa_file_list:
        faa_path, faa_basename, faa_ext = sep_path_basename_ext(each_faa_file)
        gnm_set.add(faa_basename)

    gnm_id_list_sorted = sorted([i for i in gnm_set])

    #################### check genome grouping files ####################

    group_to_gnm_dict = dict()
    if gnm_group_txt is None:
        group_to_gnm_dict['group_1'] = set()
        for each_gnm in gnm_id_list_sorted:
            group_to_gnm_dict['group_1'].add(each_gnm)
    else:
        if os.path.isfile(gnm_group_txt) is False:
            print('Specified %s not found, program exited!' % gnm_group_txt)
            exit()
        else:
            for each_gnm in open(gnm_group_txt):
                each_gnm_split = each_gnm.strip().split('\t')
                gnm_id = each_gnm_split[0]
                domain_name = each_gnm_split[1]
                if gnm_id in gnm_set:
                    if domain_name not in group_to_gnm_dict:
                        group_to_gnm_dict[domain_name] = {gnm_id}
                    else:
                        group_to_gnm_dict[domain_name].add(gnm_id)

    ############################################## check file/folder name ##############################################

    blastp_cmd_txt                          = '%s/s01_blast_cmds_%s.txt'            % (op_dir, (len(marker_seq_list)*len(faa_file_list)))
    pwd_combined_protein                    = '%s/combined.faa'                     % op_dir
    blast_op_dir                            = '%s/s01_blast'                        % op_dir
    best_hit_id_by_marker_dir               = '%s/s02_marker_id'                    % op_dir
    best_hit_seq_by_marker_dir              = '%s/s03_marker_seq'                   % op_dir
    best_hit_seq_by_marker_dir_renamed      = '%s/s04_marker_seq_by_genome_name'    % op_dir
    best_hit_aln_by_marker_dir              = '%s/s05_marker_aln'                   % op_dir
    best_hit_aln_by_marker_dir_trimmed      = '%s/s06_marker_aln_trimal'            % op_dir
    if trim_with_bmge is True:
        best_hit_aln_by_marker_dir_trimmed  = '%s/s06_marker_aln_BMGE'              % op_dir
    assess_marker_pa_dir                    = '%s/s07_assess_marker_PA'             % op_dir
    trimmed_msa_PA_concatenated_dir         = '%s/s08_iqtree_wd'                    % op_dir
    iqtree_dir                              = '%s/s08_iqtree_wd'                    % op_dir
    iqtree_cmd_txt                          = '%s/iqtree_cmds.txt'                  % iqtree_dir

    ####################################################################################################################

    # create folder
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()

    os.system('mkdir %s' % op_dir)
    os.system('mkdir %s' % blast_op_dir)

    # get blastp command
    blast_cmd_list = []
    blast_op_to_cmd_dict = dict()
    blastp_cmd_txt_handle = open(blastp_cmd_txt, 'w')
    for gnm_id in gnm_id_list_sorted:
        for each_cog in marker_id_set:
            pwd_blast_op = '%s/%s_vs_%s_blastp.txt'                                                  % (blast_op_dir, gnm_id, each_cog)

            blast_cmd   = 'blastp -subject %s/%s.%s -evalue %s -outfmt 6 -query %s/%s.%s -out %s' % (marker_seq_dir, each_cog, marker_seq_ext, e_value, faa_file_dir, gnm_id, faa_file_ext, pwd_blast_op)
            if run_psiblast is True:
                blast_cmd = 'psiblast -subject %s/%s.%s -evalue %s -outfmt 6 -query %s/%s.%s -out %s' % (marker_seq_dir, each_cog, marker_seq_ext, e_value, faa_file_dir, gnm_id, faa_file_ext, pwd_blast_op)

            blast_op_to_cmd_dict[pwd_blast_op] = blast_cmd
            blastp_cmd_txt_handle.write(blast_cmd + '\n')
            blast_cmd_list.append(blast_cmd)
    blastp_cmd_txt_handle.close()

    # run blastp
    if force_overwrite is True:
        exe_cmds(blast_cmd_list, num_of_threads)
    else:
        cmds_to_rerun = []
        num_of_good_ones = 0
        for each_blast_op in blast_op_to_cmd_dict:

            look_good = False
            if os.path.isfile(each_blast_op) is True:
                look_good = True
                num_of_good_ones += 1

            if look_good is False:
                cmds_to_rerun.append(blast_op_to_cmd_dict[each_blast_op])

        print('Detected blastp outputs: %s' % num_of_good_ones)
        exe_cmds(cmds_to_rerun, num_of_threads)

    # get best_hit_dict_by_marker
    best_hit_to_gnm_dict = dict()
    best_hit_dict_by_marker = dict()
    for gnm_id in gnm_id_list_sorted:
        for each_cog in marker_id_set:
            current_blastp_op = '%s/%s_vs_%s_blastp.txt' % (blast_op_dir, gnm_id, each_cog)
            # get best hit
            if os.path.isfile(current_blastp_op) is True:
                best_hit_gene = ''
                best_hit_score = 0
                for each_line in open(current_blastp_op):
                    each_line_split = each_line.strip().split('\t')
                    query_id = each_line_split[0]
                    bit_score = float(each_line_split[11])
                    if bit_score > best_hit_score:
                        best_hit_score = bit_score
                        best_hit_gene = query_id

                if best_hit_gene != '':
                    best_hit_to_gnm_dict[best_hit_gene] = gnm_id

                    if each_cog not in best_hit_dict_by_marker:
                        best_hit_dict_by_marker[each_cog] = [best_hit_gene]
                    else:
                        best_hit_dict_by_marker[each_cog].append(best_hit_gene)

    # create dir
    if os.path.isdir(best_hit_id_by_marker_dir) is False:
        os.system('mkdir %s' % best_hit_id_by_marker_dir)
    if os.path.isdir(best_hit_seq_by_marker_dir) is False:
        os.system('mkdir %s' % best_hit_seq_by_marker_dir)
    if os.path.isdir(best_hit_seq_by_marker_dir_renamed) is False:
        os.system('mkdir %s' % best_hit_seq_by_marker_dir_renamed)
    if os.path.isdir(best_hit_aln_by_marker_dir) is False:
        os.system('mkdir %s' % best_hit_aln_by_marker_dir)
    if os.path.isdir(best_hit_aln_by_marker_dir_trimmed) is False:
        os.system('mkdir %s' % best_hit_aln_by_marker_dir_trimmed)

    os.system('cat %s/*.%s > %s' % (faa_file_dir, faa_file_ext, pwd_combined_protein))

    processing_index = 1
    for each_marker in best_hit_dict_by_marker:
        print('Processing (extract sequence, align and trim) marker %s/%s: %s' % (processing_index, len(best_hit_dict_by_marker), each_marker))
        processing_index += 1

        current_m_hit_list      = best_hit_dict_by_marker[each_marker]
        marker_hits_txt         = ('%s/%s.txt' % (best_hit_id_by_marker_dir,          each_marker)).replace(':', '')
        marker_hits_seq         = ('%s/%s.fa'  % (best_hit_seq_by_marker_dir,         each_marker)).replace(':', '')
        marker_hits_seq_renamed = ('%s/%s.fa'  % (best_hit_seq_by_marker_dir_renamed, each_marker)).replace(':', '')
        marker_hits_aln         = ('%s/%s.aln' % (best_hit_aln_by_marker_dir,         each_marker)).replace(':', '')
        marker_hits_aln_trimmed = ('%s/%s.aln' % (best_hit_aln_by_marker_dir_trimmed, each_marker)).replace(':', '')

        with open(marker_hits_txt, 'w') as marker_hits_txt_handle:
            marker_hits_txt_handle.write('\n'.join(current_m_hit_list))

        # extract sequences
        select_seq(pwd_combined_protein, marker_hits_txt, 1, marker_hits_seq, True, False)

        # rename sequences
        marker_hits_seq_renamed_handle = open(marker_hits_seq_renamed, 'w')
        for each_seq in SeqIO.parse(marker_hits_seq, 'fasta'):
            seq_id = each_seq.id
            seq_gnm = best_hit_to_gnm_dict[seq_id]
            marker_hits_seq_renamed_handle.write('>%s\n' % seq_gnm)
            marker_hits_seq_renamed_handle.write('%s\n' % str(each_seq.seq))
        marker_hits_seq_renamed_handle.close()

        # run mafft-einsi
        mafft_cmd = 'mafft-einsi --thread %s --quiet %s  > %s' % (num_of_threads, marker_hits_seq_renamed, marker_hits_aln)
        os.system(mafft_cmd)

        # trim msa
        trim_cmd = 'trimal -in %s -out %s -automated1' % (marker_hits_aln, marker_hits_aln_trimmed)
        if trim_with_bmge is True:
            trim_cmd = 'java -jar %s -i %s -m %s -t AA -h %s -of %s' % (pwd_bmge_jar, marker_hits_aln, bmge_trim_model, bmge_entropy_score_cutoff, marker_hits_aln_trimmed)
        os.system(trim_cmd)

    ########## Assess marker by PA ##########

    present_pct_cutoff_list = [int(i) for i in pa_cutoff_str.split(',')]

    # create output dir
    if os.path.isdir(assess_marker_pa_dir) is True:
        os.system('rm -r %s' % assess_marker_pa_dir)
    os.system('mkdir %s' % assess_marker_pa_dir)

    AssessMarkerPA(best_hit_aln_by_marker_dir_trimmed, gnm_set, group_to_gnm_dict, present_pct_cutoff_list, assess_marker_pa_dir)

    ########## concatenate marker ##########

    # create output dir
    if os.path.isdir(trimmed_msa_PA_concatenated_dir) is True:
        os.system('rm -r %s' % trimmed_msa_PA_concatenated_dir)
    os.system('mkdir %s' % trimmed_msa_PA_concatenated_dir)

    qualified_cutoff_list = []
    for each_c in present_pct_cutoff_list:
        current_cutoff_trimmed_msa_dir = '%s/marker_PA%s' % (assess_marker_pa_dir, each_c)
        trimmed_msa_re   = '%s/*.aln' % current_cutoff_trimmed_msa_dir
        trimmed_msa_list = [os.path.basename(file_name) for file_name in glob.glob(trimmed_msa_re)]

        if len(trimmed_msa_list) < minimal_marker_number:
            print('The number of qualified marker under PA cutoff %s: %s, skipped!' % (each_c, len(trimmed_msa_list)))
        else:
            qualified_cutoff_list.append(each_c)
            pwd_concatenated_marker_phy       = '%s/marker_pa%s.phy'           % (trimmed_msa_PA_concatenated_dir, each_c)
            pwd_concatenated_marker_fasta     = '%s/marker_pa%s.fasta'         % (trimmed_msa_PA_concatenated_dir, each_c)
            pwd_concatenated_marker_partition = '%s/marker_pa%s_partition.txt' % (trimmed_msa_PA_concatenated_dir, each_c)
            catfasta2phy(current_cutoff_trimmed_msa_dir, 'aln', pwd_concatenated_marker_phy, pwd_concatenated_marker_fasta, pwd_concatenated_marker_partition)

    ########## get guide tree and C60+PMSF tree for each set of marker set ##########

    iqtree_cmd_txt_handle = open(iqtree_cmd_txt, 'w')
    iqtree_cmd_list = []
    for each_c in qualified_cutoff_list:
        os.system('mkdir %s/PA%s_guide_tree'    % ((iqtree_dir, each_c)))
        os.system('mkdir %s/PA%s_PMSF_C60_tree' % ((iqtree_dir, each_c)))

        msa_to_use = 'marker_pa%s.fasta' % each_c

        get_guide_tree_cmd = 'iqtree2 --seqtype AA -B 1000 --alrt 1000 --quiet -T %s -s %s --prefix PA%s_guide_tree/PA%s_guide_tree -m LG'                                                       % (num_of_threads, msa_to_use, each_c, each_c)
        get_c60_tree_cmd   = 'iqtree2 --seqtype AA -B 1000 --alrt 1000 --quiet -T %s -s %s --prefix PA%s_PMSF_C60_tree/PA%s_PMSF_C60 -m LG+C60+F+G -ft PA%s_guide_tree/PA%s_guide_tree.treefile' % (num_of_threads, msa_to_use, each_c, each_c, each_c, each_c)
        cmds_in_one_line   = '%s; %s' % (get_guide_tree_cmd, get_c60_tree_cmd)
        iqtree_cmd_txt_handle.write('%s\n' % cmds_in_one_line)
        iqtree_cmd_list.append(cmds_in_one_line)
    iqtree_cmd_txt_handle.close()

    # run iqtree
    os.chdir(iqtree_dir)
    for each_cmd in iqtree_cmd_list:
        print('Running: %s' % each_cmd)
        os.system(each_cmd)

    print('Done!')


if __name__ == '__main__':

    # initialize the options parser
    parser = argparse.ArgumentParser()
    parser.add_argument('-i',               required=True,                              help='faa dir')
    parser.add_argument('-x',               required=False, default='faa',              help='faa file extension, default: faa')
    parser.add_argument('-m',               required=True,                              help='marker seq dir, file extension need to be faa')
    parser.add_argument('-mx',              required=False, default='faa',              help='marker seq file extension, default: faa')
    parser.add_argument('-g',               required=False, default=None,               help='genome group')
    parser.add_argument('-c',               required=False, default='85',               help='presence-absence cutoffs, default: 85')
    parser.add_argument('-o',               required=True,                              help='output dir')
    parser.add_argument('-e',               required=False, default='1e-30',            help='e-value cutoff, default: 1e-30')
    parser.add_argument('-t',               required=True,  type=int,                   help='num of threads')
    parser.add_argument('-mmn',             required=False, default=1, type=int,        help='minimal marker number, default: 1')
    parser.add_argument('-psiblast',        required=False, action="store_true",        help='run psiblast')
    parser.add_argument('-bmge',            required=False, action="store_true",        help='trim MSA with BMGE, default is trimal')
    parser.add_argument('-bmge_m',          required=False, default='BLOSUM30',         help='BMGE trim model, default: BLOSUM30')
    parser.add_argument('-bmge_esc',        required=False, default='0.55',             help='BMGE entropy score cutoff, default: 0.55')
    parser.add_argument('-f',               required=False, action="store_true",        help='force overwrite')
    args = vars(parser.parse_args())
    MarkerRef2Tree(args)



import os
import argparse
from ete3 import Tree


mcmc2tree_usage = '''
============ mcmc2tree example commands ============

TreeSAK mcmc2tree -i mcmctree.out -o renamed.tree

====================================================
'''


def mcmc2tree(args):

    mamctree_out = args['i']
    tree_file    = args['o']

    if os.path.isfile(mamctree_out) is False:
        print('%s not found, program exited!' % mamctree_out)

    # get tree string from mcmctree_out
    tree_str = ''
    tree_line = 0
    current_line = 1
    for each_line in open(mamctree_out):
        if 'Species tree for FigTree.  Branch lengths = posterior mean times; 95% CIs = labels' in each_line:
            tree_line = current_line + 1
        if tree_line == current_line:
            tree_str = each_line.strip()
        current_line += 1

    tree_str_no_space = tree_str.replace(' ', '')

    # rename tree nodes
    t = Tree(tree_str_no_space, format=1)
    for each_node in t.traverse():
        if each_node.is_leaf():
            node_name_new = '_'.join(each_node.name.split('_')[1:])
        else:
            node_name_new = 't_n%s' % each_node.name
        each_node.name = node_name_new

    tree_str_renamed = t.write(format=8)

    tree_file_handle = open(tree_file, 'w')
    tree_file_handle.write(tree_str_renamed + '\n')
    tree_file_handle.close()


if __name__ == '__main__':

    mcmc2tree_parser = argparse.ArgumentParser()
    mcmc2tree_parser.add_argument('-i',  required=True,  help='the .out file from mcmctree')
    mcmc2tree_parser.add_argument('-o',  required=True,  help='output tree file')
    args = vars(mcmc2tree_parser.parse_args())
    mcmc2tree(args)
